import os

from zrb.builtin.generator.common.task_input import (
    package_author_email_input,
    package_author_name_input,
    package_description_input,
    package_documentation_input,
    package_homepage_input,
    package_repository_input,
    project_dir_input,
)
from zrb.builtin.group import plugin_group
from zrb.config.config import version
from zrb.runner import runner
from zrb.task.cmd_task import CmdTask
from zrb.task.resource_maker import ResourceMaker
from zrb.task_input.str_input import StrInput

CURRENT_DIR = os.path.dirname(__file__)

plugin_package_name_input = StrInput(
    name="package-name",
    shortcut="p",
    description="Package name",
    prompt="Package name",
    default="{{ input.project_dir }}",
)


###############################################################################
# Task Definitions
###############################################################################

copy_resource = ResourceMaker(
    name="copy-resource",
    inputs=[
        project_dir_input,
        plugin_package_name_input,
        package_description_input,
        package_homepage_input,
        package_repository_input,
        package_documentation_input,
        package_author_name_input,
        package_author_email_input,
    ],
    replacements={
        "zrbPackageName": "{{input.package_name}}",
        "zrbPackageDescription": "{{input.package_description}}",
        "zrbPackageHomepage": "{{input.package_homepage}}",
        "zrbPackageRepository": "{{input.package_repository}}",
        "zrbPackageDocumentation": "{{input.package_documentation}}",
        "zrbPackageAuthorName": "{{input.package_author_name}}",
        "zrbPackageAuthorEmail": "{{input.package_author_email}}",
        "zrbVersion": version,
    },
    template_path=os.path.join(CURRENT_DIR, "template"),
    destination_path="{{ input.project_dir }}",
    excludes=[
        "*/__pycache__",
    ],
)

create = CmdTask(
    name="create",
    description="Create plugin",
    group=plugin_group,
    upstreams=[copy_resource],
    inputs=[project_dir_input],
    cmd=[
        "set -e",
        'cd "{{input.project_dir}}"',
        "if [ ! -d .git ]",
        "then",
        "  echo Initialize project git repository",
        "  git init",
        "fi",
        'echo "Happy coding :)"',
    ],
)
runner.register(create)



from zrb import Task, runner
from zrb.builtin.group import project_group

destroy_project = Task(
    name="destroy",
    group=project_group,
    upstreams=[],
    description="Remove project deployment",
    run=lambda *args, **kwargs: kwargs.get("_task").print_out("🆗"),
)
runner.register(destroy_project)



"""A Kubernetes Python Pulumi program to deploy kebab-zrb-app-name"""

import os

import pulumi
from pulumi_kubernetes.helm.v3 import Chart, LocalChartOpts

CURRENT_DIR = os.path.dirname(__file__)
NAMESPACE = os.getenv("NAMESPACE", "default")

image = os.getenv("IMAGE", "kebab-zrb-app-name:latest")
replica = int(os.getenv("REPLICA", "1"))
app_labels = {"app": "kebab-zrb-app-name"}

# https://www.pulumi.com/registry/packages/kubernetes/api-docs/helm/v3/chart/#local-chart-directory  # noqa
release = Chart(
    "kebab-zrb-app-name",
    LocalChartOpts(
        path="./helm-charts/kebab-zrb-app-name", namespace=NAMESPACE, values={}
    ),
)

pulumi.export("helm-release", release.resources)



import os

from zrb import CmdTask, Env, EnvFile, PathChecker, runner
from zrb.builtin.group import project_group

from ._config import (
    APP_FRONTEND_BUILD_DIR,
    APP_FRONTEND_DIR,
    APP_TEMPLATE_ENV_FILE_NAME,
    CURRENT_DIR,
)

###############################################################################
# ⚙️ build-kebab-zrb-task-name-frontend
###############################################################################

build_snake_zrb_app_name_frontend = CmdTask(
    icon="🚤",
    name="build-kebab-zrb-app-name-frontend",
    description="Build frontend for human readable zrb app name",
    group=project_group,
    cwd=APP_FRONTEND_DIR,
    cmd_path=os.path.join(CURRENT_DIR, "cmd", "app-build-frontend.sh"),
    checkers=[
        PathChecker(
            name="check-kebab-zrb-app-name-frontend-build", path=APP_FRONTEND_BUILD_DIR
        )
    ],
    env_files=[
        EnvFile(path=APP_TEMPLATE_ENV_FILE_NAME, prefix="ZRB_ENV_PREFIX"),
    ],
    envs=[Env(name="WATCH", os_name="", default="1")],
)
runner.register(build_snake_zrb_app_name_frontend)

###############################################################################
# ⚙️ build-kebab-zrb-task-name-frontend-once
###############################################################################

build_snake_zrb_app_name_frontend_once = CmdTask(
    icon="🚤",
    name="build-kebab-zrb-app-name-frontend_once",
    description="Build frontend for human readable zrb app name",
    cwd=APP_FRONTEND_DIR,
    cmd_path=os.path.join(CURRENT_DIR, "cmd", "app-build-frontend.sh"),
    checkers=[
        PathChecker(
            name="check-kebab-zrb-app-name-frontend-build", path=APP_FRONTEND_BUILD_DIR
        )
    ],
    env_files=[
        EnvFile(path=APP_TEMPLATE_ENV_FILE_NAME, prefix="ZRB_ENV_PREFIX"),
    ],
    envs=[Env(name="WATCH", os_name="", default="0")],
)



from logging import Logger

from component.messagebus import Publisher
from component.rpc import Caller
from fastapi import FastAPI
from module.auth.component import Authorizer
from module.auth.entity.group.api import register_api as register_group_api
from module.auth.entity.permission.api import register_api as register_permission_api
from module.auth.entity.user.api import register_api as register_user_api
from module.auth.entity.user.api import register_auth_api as register_user_login_api


def register_api(
    logger: Logger,
    app: FastAPI,
    authorizer: Authorizer,
    rpc_caller: Caller,
    publisher: Publisher,
):
    logger.info('🥪 Register API for "auth"')
    register_user_login_api(logger, app, authorizer, rpc_caller, publisher)
    register_permission_api(logger, app, authorizer, rpc_caller, publisher)
    register_group_api(logger, app, authorizer, rpc_caller, publisher)
    register_user_api(logger, app, authorizer, rpc_caller, publisher)



from component.repo import DBEntityMixin, DBRepo, Repo
from module.snake_zrb_module_name.integration import Base
from module.snake_zrb_module_name.schema.snake_zrb_entity_name import (
    PascalZrbEntityName,
    PascalZrbEntityNameData,
)
from sqlalchemy import Column, String


class DBEntityPascalZrbEntityName(Base, DBEntityMixin):
    class Config:
        orm_mode = True
        from_attributes = True

    __tablename__ = "snake_zrb_plural_entity_name"
    snake_zrb_column_name = Column(String)


class PascalZrbEntityNameRepo(Repo[PascalZrbEntityName, PascalZrbEntityNameData]):
    pass


class PascalZrbEntityNameDBRepo(
    DBRepo[PascalZrbEntityName, PascalZrbEntityNameData], PascalZrbEntityNameRepo
):
    schema_cls = PascalZrbEntityName
    db_entity_cls = DBEntityPascalZrbEntityName



import os
from typing import Mapping

import jsons

from zrb import Env, EnvFile, ServiceConfig
from zrb.helper.util import to_kebab_case

CURRENT_DIR = os.path.dirname(__file__)
PROJECT_DIR = os.path.abspath(os.path.join(CURRENT_DIR, "..", ".."))
RESOURCE_DIR = os.path.join(PROJECT_DIR, "src", "kebab-zrb-app-name")
DEPLOYMENT_DIR = os.path.join(RESOURCE_DIR, "deployment")
DEPLOYMENT_TEMPLATE_ENV_FILE_NAME = os.path.join(DEPLOYMENT_DIR, "template.env")
APP_DIR = os.path.join(RESOURCE_DIR, "src")
APP_FRONTEND_DIR = os.path.join(APP_DIR, "frontend")
APP_FRONTEND_BUILD_DIR = os.path.join(APP_FRONTEND_DIR, "build")
APP_TEMPLATE_ENV_FILE_NAME = os.path.join(APP_DIR, "template.env")
LOAD_TEST_DIR = os.path.join(RESOURCE_DIR, "loadtest")
LOAD_TEST_TEMPLATE_ENV_FILE_NAME = os.path.join(RESOURCE_DIR, "template.env")

MODULE_CONFIG_PATH = os.path.join(CURRENT_DIR, "config", "modules.json")

with open(MODULE_CONFIG_PATH) as file:
    MODULE_JSON_STR = file.read()
MODULES = jsons.loads(MODULE_JSON_STR)

###############################################################################
# Service Configs
###############################################################################

_OTEL_EXPORTER_ENDPOINT_ENV = Env(
    name="APP_OTEL_EXPORTER_OTLP_ENDPOINT",
    os_name="",
    default="http://otel-collector:4317",
)

_CONTAINER_ENV_PREFIX = "CONTAINER_ZRB_ENV_PREFIX"
_APP_TEMPLATE_ENV_FILE = EnvFile(
    path=APP_TEMPLATE_ENV_FILE_NAME, prefix=_CONTAINER_ENV_PREFIX
)
_DOCKER_COMPOSE_APP_ENV_FILE_NAME = os.path.join(RESOURCE_DIR, "docker-compose-app.env")
_DOCKER_COMPOSE_APP_ENV_FILE = EnvFile(
    path=_DOCKER_COMPOSE_APP_ENV_FILE_NAME, prefix=_CONTAINER_ENV_PREFIX
)
_DISABLE_ALL_MODULE_ENV_FILE_NAME = os.path.join(
    RESOURCE_DIR, "all-module-disabled.env"
)
_DISABLE_ALL_MODULE_ENV_FILE = EnvFile(
    path=_DISABLE_ALL_MODULE_ENV_FILE_NAME, prefix=_CONTAINER_ENV_PREFIX
)
_ENABLE_ALL_MODULE_ENV_FILE_NAME = os.path.join(RESOURCE_DIR, "all-module-enabled.env")
_ENABLE_ALL_MODULE_ENV_FILE = EnvFile(
    path=_ENABLE_ALL_MODULE_ENV_FILE_NAME, prefix=_CONTAINER_ENV_PREFIX
)

SERVICE_CONFIGS: Mapping[str, ServiceConfig] = {
    "kebab-zrb-app-name": ServiceConfig(
        env_files=[
            _APP_TEMPLATE_ENV_FILE,
            _DOCKER_COMPOSE_APP_ENV_FILE,
            _ENABLE_ALL_MODULE_ENV_FILE,
        ],
        envs=[_OTEL_EXPORTER_ENDPOINT_ENV],
    ),
    "kebab-zrb-app-name-gateway": ServiceConfig(
        env_files=[
            _APP_TEMPLATE_ENV_FILE,
            _DOCKER_COMPOSE_APP_ENV_FILE,
            _ENABLE_ALL_MODULE_ENV_FILE,
        ],
        envs=[_OTEL_EXPORTER_ENDPOINT_ENV],
    ),
}
for module in MODULES:
    service_name = f"kebab-zrb-app-name-{to_kebab_case(module)}-service"
    SERVICE_CONFIGS[service_name] = ServiceConfig(
        env_files=[
            _APP_TEMPLATE_ENV_FILE,
            _DOCKER_COMPOSE_APP_ENV_FILE,
            _DISABLE_ALL_MODULE_ENV_FILE,
        ],
        envs=[_OTEL_EXPORTER_ENDPOINT_ENV],
    )



from component.messagebus.kafka.admin import KafkaAdmin
from component.messagebus.kafka.consumer import KafkaConsumer
from component.messagebus.kafka.publisher import KafkaPublisher
from component.messagebus.messagebus import (
    Admin,
    Consumer,
    MessageSerializer,
    Publisher,
)
from component.messagebus.mock import MockAdmin, MockConsumer, MockPublisher
from component.messagebus.rabbitmq.admin import RMQAdmin
from component.messagebus.rabbitmq.consumer import RMQConsumer
from component.messagebus.rabbitmq.publisher import RMQPublisher

assert Admin
assert Consumer
assert Publisher
assert MessageSerializer
assert MockAdmin
assert MockConsumer
assert MockPublisher
assert KafkaAdmin
assert KafkaConsumer
assert KafkaPublisher
assert RMQAdmin
assert RMQConsumer
assert RMQPublisher



import datetime
from typing import Optional

from pydantic import BaseModel


class BaseDateTimeSchema(BaseModel):
    created_at: Optional[datetime.datetime] = None
    created_by: Optional[str] = None
    updated_at: Optional[datetime.datetime] = None
    updated_by: Optional[str] = None


class BaseCountSchema(BaseModel):
    count: int



from typing import List

from component.schema import BaseCountSchema, BaseDateTimeSchema


class ActivityData(BaseDateTimeSchema):
    action: str
    entity: str
    data: str


class Activity(ActivityData):
    id: str

    class Config:
        orm_mode = True
        from_attributes = True


class ActivityResult(BaseCountSchema):
    data: List[Activity]



from abc import ABC, abstractmethod
from typing import Any, Callable, List, Mapping, TypeVar

TRPCHandler = Callable[..., Any]
TMessage = TypeVar("TMessage", bound="Message")
TResult = TypeVar("TResult", bound="Result")


class Caller(ABC):
    @abstractmethod
    async def call(self, rpc_name: str, *args: Any, **kwargs: Any) -> Any:
        pass


class Server(ABC):
    @abstractmethod
    def register(self, rpc_name: str) -> Callable[[TRPCHandler], Any]:
        pass

    @abstractmethod
    def start(self):
        pass

    @abstractmethod
    def stop(self):
        pass


class Message:
    def __init__(self, reply_event: str, args: List[Any], kwargs: Mapping[str, Any]):
        self.reply_event = reply_event
        self.args = args
        self.kwargs = kwargs

    def to_dict(self):
        return {
            "args": self.args,
            "kwargs": self.kwargs,
            "reply_event": self.reply_event,
        }

    @classmethod
    def from_dict(cls, dictionary: Mapping[str, Any]) -> TMessage:
        return cls(
            reply_event=dictionary.get("reply_event", ""),
            args=dictionary.get("args", []),
            kwargs=dictionary.get("kwargs", {}),
        )


class Result:
    def __init__(self, result: Any = None, error: str = ""):
        self.result = result
        self.error = error

    def to_dict(self):
        return {"result": self.result, "error": self.error}

    @classmethod
    def from_dict(cls, dictionary: Mapping[str, Any]) -> TResult:
        return cls(
            result=dictionary.get("result", None), error=dictionary.get("error", "")
        )



import datetime
import random

from zrb.helper.python_task import show_lines
from zrb.helper.typing import Any, List
from zrb.runner import runner
from zrb.task.decorator import python_task
from zrb.task_input.int_input import IntInput
from zrb.task_input.str_input import StrInput

_MIN_WIDTH = 10
_MOTIVATIONAL_QUOTES = [
    [
        "The best time to plant a tree was 20 years ago.",
        "The second best time is now.",
        "~ Chinese Proverb",
    ],
    ["The only way to do great work is to love what you do.", "~ Steve Jobs"],
    ["Believe you can and you're halfway there.", "~ Theodore Roosevelt"],
    ["It does not matter how slowly you go as long as you do not stop.", "~ Confucius"],
    ["Everything you've ever wanted is on the other side of fear.", "~ George Addair"],
    [
        "Success is not final, failure is not fatal:",
        "It is the courage to continue that counts.",
        "~ Winston Churchill",
    ],
    [
        "Hardships often prepare ordinary people",
        "for an extraordinary destiny.",
        "~ C.S. Lewis",
    ],
    [
        "Your time is limited, don't waste it living someone else's life.",
        "~ Steve Jobs",
    ],
    ["Don’t watch the clock; do what it does. Keep going.", "~ Sam Levenson"],
    [
        "You are never too old to set another goal or to dream a new dream.",
        "~ C.S. Lewis",
    ],
    [
        "The only limit to our realization of tomorrow",
        "will be our doubts of today.",
        "~ Franklin D. Roosevelt",
    ],
    [
        "Believe in yourself.",
        "You are braver than you think, more talented than you know,"
        "and capable of more than you imagine.",
        "~ Roy T. Bennett",
    ],
    [
        "I can't change the direction of the wind,",
        "but I can adjust my sails to always reach my destination.",
        "~ Jimmy Dean",
    ],
    ["You are enough just as you are.", "~ Meghan Markle"],
    [
        "The future belongs to those",
        "who believe in the beauty of their dreams.",
        "~ Eleanor Roosevelt",
    ],
]


@python_task(
    name="say",
    inputs=[StrInput(name="text", default=""), IntInput(name="width", default=80)],
    description="Say anything, https://www.youtube.com/watch?v=MbPr1oHO4Hw",
    runner=runner,
)
def say(*args: Any, **kwargs: Any):
    width: int = kwargs.get("width", 80)
    if width < _MIN_WIDTH:
        width = _MIN_WIDTH
    text: str = kwargs.get("text", "")
    top_border = "┌" + "─" * (width + 2) + "┐"
    content = ["| " + line + " |" for line in _get_content(text, width)]
    bottom_border = "└" + "─" * (width + 2) + "┘"
    lines = (
        [top_border]
        + content
        + [bottom_border]
        + [
            "      \\",
            "       \\",
            "  o    ___    o",
            "  | ┌-------┐ |",
            "  |(| o   o |)|",
            "    | └---┘ |",
            "    └-------┘",
        ]
    )
    show_lines(kwargs["_task"], *lines)


def _get_content(text: str, width: int) -> List[str]:
    if text == "":
        now = datetime.datetime.now()
        today = "Today is " + now.strftime("%A, %B %d, %Y")
        current_time = "Current time is " + now.strftime("%I:%M %p")
        motivational_quote = random.choice(_MOTIVATIONAL_QUOTES)
        return [
            today.ljust(width),
            current_time.ljust(width),
            "".ljust(width),
        ] + [line.ljust(width) for line in motivational_quote]
    return _split_text_by_width(text, width)


def _split_text_by_width(text: str, width: int) -> List[str]:
    original_lines = text.split("\n")
    new_lines = []
    for original_line in original_lines:
        new_lines += [
            original_line[i : i + width].ljust(width)
            for i in range(0, len(original_line), width)
        ]
    return new_lines



import os

from zrb.helper.typecheck import typechecked


@typechecked
class PidModel:
    def __init__(self):
        self.__task_pid: int = os.getpid()

    def _set_task_pid(self, pid: int):
        self.__task_pid = pid

    def _get_task_pid(self) -> int:
        return self.__task_pid



from zrb.helper.typing import Any, Callable
from zrb.task.any_task import AnyTask

_TaskEventHandler = Callable[[AnyTask], Any]
OnTriggered = _TaskEventHandler
OnWaiting = _TaskEventHandler
OnSkipped = _TaskEventHandler
OnStarted = _TaskEventHandler
OnReady = _TaskEventHandler
OnRetry = _TaskEventHandler
OnFailed = Callable[[AnyTask, bool, Exception], Any]



import os

from zrb.builtin.generator.common.helper import (
    validate_existing_project_dir,
    validate_inexisting_automation,
)
from zrb.builtin.generator.common.task_factory import create_register_module
from zrb.builtin.generator.common.task_input import project_dir_input, task_name_input
from zrb.builtin.group import project_add_group
from zrb.helper.typing import Any
from zrb.runner import runner
from zrb.task.decorator import python_task
from zrb.task.resource_maker import ResourceMaker
from zrb.task.task import Task

CURRENT_DIR = os.path.dirname(__file__)

###############################################################################
# Task Definitions
###############################################################################


@python_task(
    name="validate",
    inputs=[
        project_dir_input,
        task_name_input,
    ],
)
async def validate(*args: Any, **kwargs: Any):
    project_dir = kwargs.get("project_dir")
    validate_existing_project_dir(project_dir)
    task_name = kwargs.get("task_name")
    validate_inexisting_automation(project_dir, task_name)


copy_resource = ResourceMaker(
    name="copy-resource",
    inputs=[
        project_dir_input,
        task_name_input,
    ],
    upstreams=[validate],
    replacements={
        "zrbTaskName": "{{input.task_name}}",
    },
    template_path=os.path.join(CURRENT_DIR, "template"),
    destination_path="{{ input.project_dir }}",
    excludes=[
        "*/__pycache__",
    ],
)

register_module = create_register_module(
    module_path="_automate.{{util.to_snake_case(input.task_name)}}",
    inputs=[task_name_input],
    upstreams=[copy_resource],
)


@python_task(
    name="cmd-task", group=project_add_group, upstreams=[register_module], runner=runner
)
async def add_cmd_task(*args: Any, **kwargs: Any):
    task: Task = kwargs.get("_task")
    task.print_out("Success")



from zrb import PortChecker

snake_zrb_app_name_checker = PortChecker(
    name="check-kebab-zrb-app-name",
    host="{{input.snake_zrb_app_name_host}}",
    port="{{env.APP_PORT}}",
)

snake_zrb_app_name_container_checker = PortChecker(
    name="check-kebab-zrb-app-name",
    host="{{input.snake_zrb_app_name_host}}",
    port="{{env.HOST_PORT}}",
)



from zrb.helper.typecheck import typechecked
from zrb.helper.typing import Any, Optional, Union
from zrb.task_input.base_input import BaseInput

# flake8: noqa E501


@typechecked
class PasswordInput(BaseInput):
    """
    A specialized input class for handling password or sensitive data inputs in tasks.

    `PasswordInput` extends `BaseInput` to manage inputs that should be treated as sensitive,
    such as passwords. It ensures that the input is hidden when entered, providing an added
    layer of security and privacy. This class supports various features like default values,
    prompts, and flags, tailored for the safe handling of sensitive information.

    Args:
        name (str): The name of the input, serving as an identifier.
        shortcut (Optional[str]): An optional shortcut string for the input.
        default (Optional[Any]): The default value for the input, expected to be a string if set.
        description (Optional[str]): A brief description of the input's purpose.
        show_default (Union[bool, str, None]): Option to display the default value. Can be a boolean or string.
        prompt (Union[bool, str]): A boolean or string to prompt the user for input. If `True`, uses the default prompt.
        confirmation_prompt (Union[bool, str]): If `True`, the user is asked to confirm the input.
        prompt_required (bool): If `True`, the prompt for input is mandatory.
        is_flag (Optional[bool]): Indicates if the input is a flag. If `True`, the input accepts boolean flag values.
        flag_value (Optional[Any]): The value associated with the flag if `is_flag` is `True`.
        multiple (bool): If `True`, allows multiple values for the input.
        count (bool): If `True`, counts the occurrences of the input.
        allow_from_autoenv (bool): If `True`, allows the input to be automatically populated from the environment.
        hidden (bool): If `True`, the input is not shown in help messages or documentation.
        show_choices (bool): If `True`, displays the available choices to the user (if applicable).
        show_envvar (bool): Indicates whether to display the environment variable associated with this input.
        nargs (int): The number of arguments that the input can accept.
        should_render (bool): If `True’, the input is rendered in the UI or command-line interface.

    Examples:
        >>> password_input = PasswordInput(name='password', description='Enter your password')
        >>> password_input.is_hidden()
        True
    """

    def __init__(
        self,
        name: str,
        shortcut: Optional[str] = None,
        default: Optional[Any] = None,
        description: Optional[str] = None,
        show_default: Union[bool, str, None] = None,
        prompt: Union[bool, str] = True,
        confirmation_prompt: Union[bool, str] = False,
        prompt_required: bool = True,
        is_flag: Optional[bool] = None,
        flag_value: Optional[Any] = None,
        multiple: bool = False,
        count: bool = False,
        allow_from_autoenv: bool = True,
        hidden: bool = False,
        show_choices: bool = True,
        show_envvar: bool = False,
        nargs: int = 1,
        should_render: bool = True,
    ):
        BaseInput.__init__(
            self,
            name=name,
            shortcut=shortcut,
            default=default,
            description=description,
            show_default=show_default,
            prompt=prompt,
            confirmation_prompt=confirmation_prompt,
            prompt_required=prompt_required,
            hide_input=True,
            is_flag=is_flag,
            flag_value=flag_value,
            multiple=multiple,
            count=count,
            allow_from_autoenv=allow_from_autoenv,
            type=str,
            hidden=hidden,
            show_choices=show_choices,
            show_envvar=show_envvar,
            nargs=nargs,
            should_render=should_render,
        )

    def is_hidden(self) -> bool:
        return True



from zrb.builtin.group import env_group
from zrb.helper.accessories.color import colored
from zrb.helper.typing import Any, List
from zrb.runner import runner
from zrb.task.decorator import python_task
from zrb.task.task import Task

###############################################################################
# Task Definitions
###############################################################################


@python_task(
    name="get", group=env_group, description="Get environment values", runner=runner
)
async def get(*args: Any, **kwargs: Any):
    task: Task = kwargs["_task"]
    env_map = task.get_env_map()
    names = list(env_map.keys())
    names.sort()
    colored_equal = colored("=", color="grey", attrs=["dark"])
    env_lines: List[str] = []
    for name in names:
        value = env_map[name]
        colored_name = colored(name, color="green", attrs=["bold"])
        colored_value = colored(value, attrs=["bold"])
        env_lines.append(f"{colored_name}{colored_equal}{colored_value}")
    line_separator = "\n    "
    task.print_out(line_separator + line_separator.join(env_lines))
    return env_map



from zrb.builtin.helper.reccuring_action import create_recurring_action
from zrb.runner import runner
from zrb.task.recurring_task import RecurringTask
from zrb.task.time_watcher import TimeWatcher
from zrb.task_input.str_input import StrInput

schedule = RecurringTask(
    name="schedule",
    icon="📅",
    color="yellow",
    description="Show message/run command periodically",
    inputs=[
        StrInput(
            name="schedule",
            default="* * * * *",
            prompt="Schedule cron pattern (minute hour day(month) month day(week)",  # noqa
            description="Schedule cron pattern to show the message",
        ),
    ],
    triggers=[
        TimeWatcher(
            name="watch-time", color="cyan", icon="⏰", schedule="{{input.schedule}}"
        )
    ],
    task=create_recurring_action(
        notif_title="Schedule",
        trigger_caption="Schedule",
        trigger_xcom_key="watch-time.scheduled-time",
    ),
)
runner.register(schedule)



from logging import Logger

from component.messagebus import Consumer, Publisher
from component.rpc import Caller
from module.log.entity.activity.event import register_event as register_activity_event


def register_event(
    logger: Logger, consumer: Consumer, rpc_caller: Caller, publisher: Publisher
):
    logger.info('🥪 Register event handlers for "log"')
    register_activity_event(logger, consumer, rpc_caller, publisher)



from zrb.builtin.devtool.devtool_install import (
    install_aws,
    install_docker,
    install_gcloud,
    install_gvm,
    install_helix,
    install_helm,
    install_kubectl,
    install_nvm,
    install_pyenv,
    install_sdkman,
    install_tmux,
    install_zsh,
)
from zrb.builtin.group import ubuntu_group, ubuntu_install_group
from zrb.runner import runner
from zrb.task.cmd_task import CmdTask
from zrb.task.flow_task import FlowTask

###############################################################################
# Task Definitions
###############################################################################


update = CmdTask(
    name="update",
    group=ubuntu_group,
    description="Update ubuntu",
    cmd=[
        "sudo apt update",
        "sudo apt upgrade -y",
    ],
    retry_interval=3,
    preexec_fn=None,
)
runner.register(update)

install_toys = CmdTask(
    name="toys",
    group=ubuntu_install_group,
    description="Install ubuntu toy packages",
    cmd=[
        "sudo apt install -y lolcat cowsay figlet neofetch",
    ],
    retry_interval=3,
    preexec_fn=None,
)
update_and_install_toys: CmdTask = install_toys.copy()
update_and_install_toys.add_upstream(update)
runner.register(update_and_install_toys)

install_essentials = CmdTask(
    name="essentials",
    group=ubuntu_install_group,
    description="Install ubuntu essential packages",
    cmd=[
        "sudo apt install -y \\",
        "build-essential python3-distutils libssl-dev zlib1g-dev \\"
        "libbz2-dev libreadline-dev libsqlite3-dev libpq-dev python3-dev \\",
        "llvm libncurses5-dev libncursesw5-dev xz-utils tk-dev libffi-dev \\",
        "liblzma-dev python3-openssl libblas-dev liblapack-dev rustc \\",
        "golang gfortran fd-find ripgrep wget curl git ncat zip unzip \\",
        "cmake make tree tmux zsh neovim xdotool xsel",
    ],
    retry_interval=3,
    preexec_fn=None,
)
update_and_install_essentials: CmdTask = install_essentials.copy()
update_and_install_essentials.add_upstream(update)
runner.register(update_and_install_essentials)

install_tex = CmdTask(
    name="tex",
    group=ubuntu_install_group,
    description="Install ubuntu tex packages",
    cmd=[
        "sudo apt install -y \\",
        "texlive-full texlive-latex-base texlive-fonts-recommended \\",
        "texlive-fonts-extra texlive-latex-extra",
    ],
    retry_interval=3,
    preexec_fn=None,
)
update_and_install_tex: CmdTask = install_tex.copy()
update_and_install_tex.add_upstream(update)
runner.register(update_and_install_tex)

install_all = FlowTask(
    name="all",
    group=ubuntu_install_group,
    description="Install all ubuntu packages",
    steps=[
        update,
        install_essentials,
        install_toys,
        install_tex,
        install_zsh,
        install_tmux,
        install_gvm,
        install_nvm,
        install_pyenv,
        install_sdkman,
        install_aws,
        install_gcloud,
        install_docker,
        install_kubectl,
        install_helm,
        install_helix,
    ],
)
runner.register(install_all)



from logging import Logger

from component.messagebus import Publisher
from component.rpc import Caller, Server
from module.auth.entity.group.rpc import register_rpc as register_group_rpc
from module.auth.entity.permission.rpc import register_rpc as register_permission_rpc
from module.auth.entity.user.rpc import register_rpc as register_user_rpc


def register_rpc(
    logger: Logger, rpc_server: Server, rpc_caller: Caller, publisher: Publisher
):
    logger.info('🥪 Register RPC handlers for "auth"')
    register_permission_rpc(logger, rpc_server, rpc_caller, publisher)
    register_group_rpc(logger, rpc_server, rpc_caller, publisher)
    register_user_rpc(logger, rpc_server, rpc_caller, publisher)



import os
from typing import List

from zrb import CmdTask, Env, EnvFile, HTTPChecker, Task
from zrb.helper.util import to_kebab_case, to_snake_case

from ._config import APP_DIR, APP_TEMPLATE_ENV_FILE_NAME, CURRENT_DIR, MODULES
from ._env import app_enable_otel_env
from ._helper import should_start_local_microservices
from ._input import (
    enable_monitoring_input,
    host_input,
    https_input,
    local_input,
    run_mode_input,
)


def get_start_microservices(upstreams: List[Task]) -> List[Task]:
    disable_all_module_envs = _get_disable_all_module_envs()
    start_microservices: List[Task] = []
    for module_index, module_name in enumerate(MODULES):
        kebab_module_name = to_kebab_case(module_name)
        # Define start service task
        start_service = CmdTask(
            name=f"start-kebab-zrb-app-name-{kebab_module_name}-service",
            inputs=[
                local_input,
                run_mode_input,
                host_input,
                https_input,
                enable_monitoring_input,
            ],
            should_execute=should_start_local_microservices,
            upstreams=upstreams,
            cwd=APP_DIR,
            env_files=[_get_service_env_file(module_name)],
            envs=disable_all_module_envs
            + _get_service_envs(zrbAppHttpPort, module_index, module_name),
            cmd_path=[
                os.path.join(CURRENT_DIR, "cmd", "activate-venv.sh"),
                os.path.join(CURRENT_DIR, "cmd", "app-start.sh"),
            ],
            checkers=[
                HTTPChecker(
                    name=f"check-kebab-zrb-app-name-{kebab_module_name}-service",
                    host="{{input.snake_zrb_app_name_host}}",
                    url="/readiness",
                    port="{{env.APP_PORT}}",
                    is_https="{{input.snake_zrb_app_name_https}}",
                    should_execute=should_start_local_microservices,
                )
            ],
        )
        start_microservices.append(start_service)
    return start_microservices


def _get_service_envs(base_port: int, module_index: int, module_name: str) -> List[Env]:
    kebab_module_name = to_kebab_case(module_name)
    snake_module_name = to_snake_case(module_name)
    upper_snake_module_name = snake_module_name.upper()
    # Define service env
    app_broker_type_env = Env(
        name="APP_BROKER_TYPE",
        os_name=f"ZRB_ENV_PREFIX_{upper_snake_module_name}_APP_BROKER_TYPE",
        default="rabbitmq",
    )
    app_port_env = Env(
        name="APP_PORT",
        os_name=f"ZRB_ENV_PREFIX_{upper_snake_module_name}_APP_PORT",
        default=str(base_port + module_index + 1),
    )
    enable_module_env = Env(
        name=f"APP_ENABLE_{upper_snake_module_name}_MODULE", os_name="", default="true"
    )
    zrb_app_name_env = Env(
        name="APP_NAME",
        os_name=f"ZRB_ENV_PREFIX_{upper_snake_module_name}_APP_NAME",
        default=f"kebab-zrb-app-name-{kebab_module_name}-service",
    )
    return [
        app_broker_type_env,
        app_enable_otel_env,
        app_port_env,
        zrb_app_name_env,
        enable_module_env,
    ]


def _get_service_env_file(module_name: str) -> EnvFile:
    snake_module_name = to_snake_case(module_name)
    upper_snake_module_name = snake_module_name.upper()
    return EnvFile(
        path=APP_TEMPLATE_ENV_FILE_NAME,
        prefix=f"ZRB_ENV_PREFIX_{upper_snake_module_name}",
    )


def _get_disable_all_module_envs() -> List[Env]:
    disable_all_module_envs: List[Env] = []
    for module in MODULES:
        snake_module_name = to_snake_case(module)
        upper_snake_module_name = snake_module_name.upper()
        disable_all_module_envs.append(
            Env(
                name=f"APP_ENABLE_{upper_snake_module_name}_MODULE",
                os_name="",
                default="false",
            )
        )
    return disable_all_module_envs



from zrb import CmdTask, runner
from zrb.builtin.group import project_group

###############################################################################
# ⚙️ kebab-zrb-task-name
###############################################################################

snake_zrb_task_name = CmdTask(
    name="kebab-zrb-task-name",
    description="human readable zrb task name",
    group=project_group,
    cmd=["echo human readable zrb task name"],
)
runner.register(snake_zrb_task_name)



import os

from zrb import DockerComposeTask, Env, EnvFile, HTTPChecker, runner
from zrb.builtin.group import project_group

###############################################################################
# 🏔️ Constants
###############################################################################

CURRENT_DIR = os.path.dirname(__file__)
PROJECT_DIR = os.path.abspath(os.path.join(CURRENT_DIR, ".."))
RESOURCE_DIR = os.path.join(PROJECT_DIR, "src", "kebab-zrb-task-name")

###############################################################################
# ⚙️ kebab-zrb-task-name
###############################################################################

snake_zrb_task_name = DockerComposeTask(
    name="kebab-zrb-task-name",
    description="human readable zrb task name",
    group=project_group,
    cwd=RESOURCE_DIR,
    compose_cmd="zrbComposeCommand",
    compose_env_prefix="ZRB_ENV_PREFIX",
    env_files=[
        EnvFile(
            path=os.path.join(RESOURCE_DIR, "docker-compose.env"),
            prefix="ZRB_ENV_PREFIX",
        )
    ],
    envs=[
        Env(
            name="HOST_PORT", os_name="ZRB_ENV_PREFIX_HOST_PORT", default="zrbHttpPort"
        ),
    ],
    checkers=[HTTPChecker(name="check-kebab-zrb-task-name", port="{{env.HOST_PORT}}")],
)
runner.register(snake_zrb_task_name)



import os
from typing import Any

from zrb import IntInput, ResourceMaker, StrInput, Task, python_task, runner
from zrb.builtin.generator.common.helper import (
    validate_existing_project_dir,
    validate_inexisting_automation,
)
from zrb.builtin.generator.common.task_factory import create_register_module
from zrb.builtin.generator.common.task_input import (
    app_image_name_input,
    env_prefix_input,
    project_dir_input,
)
from zrb.builtin.generator.project_task.task_factory import (
    create_add_deploy_upstream,
    create_add_destroy_upstream,
    create_add_remove_containers_upstream,
    create_add_start_containers_upstream,
    create_add_start_upstream,
    create_add_stop_containers_upstream,
    create_ensure_project_tasks,
)
from zrb.builtin.group import project_add_group

CURRENT_DIR = os.path.dirname(__file__)
SNAKE_APP_NAME_TPL = "{{util.to_snake_case(input.app_name)}}"
KEBAB_APP_NAME_TPL = "{{util.to_kebab_case(app_name)}}"

###############################################################################
# Task Inputs
###############################################################################

app_name_input = StrInput(
    name="app-name",
    shortcut="a",
    description="App name",
    prompt="App name",
    default="zrbMetaTemplateName",
)

app_port_input = IntInput(
    name="app-port",
    shortcut="p",
    description="HTTP port",
    prompt="HTTP port",
    default=zrbMetaDefaultAppPort,
)

###############################################################################
# Task Definitions
###############################################################################


@python_task(
    name="validate",
    inputs=[project_dir_input, app_name_input],
)
async def validate(*args: Any, **kwargs: Any):
    project_dir = kwargs.get("project_dir")
    validate_existing_project_dir(project_dir)
    app_name = kwargs.get("app_name")
    validate_inexisting_automation(project_dir, app_name)
    source_dir = os.path.join(project_dir, "src", f"{KEBAB_APP_NAME_TPL}")
    if os.path.exists(source_dir):
        raise Exception(f"Source already exists: {source_dir}")


copy_resource = ResourceMaker(
    name="copy-resource",
    inputs=[
        project_dir_input,
        app_name_input,
        app_image_name_input,
        app_port_input,
        env_prefix_input,
    ],
    upstreams=[validate],
    replacements={
        "zrbAppName": "{{input.app_name}}",
        "zrbAppPort": '{{util.coalesce(input.app_port, "8080")}}',
        "ZRB_ENV_PREFIX": '{{util.coalesce(input.env_prefix, "MY").upper()}}',
        "zrb-app-image-name": "{{input.app_image_name}}",
    },
    template_path=os.path.join(CURRENT_DIR, "template"),
    destination_path="{{ input.project_dir }}",
    excludes=[
        "*/deployment/venv",
        "*/__pycache__",
    ],
)

register_local_module = create_register_module(
    module_path=f"_automate.{SNAKE_APP_NAME_TPL}.local",
    alias=f"{SNAKE_APP_NAME_TPL}_local",
    inputs=[app_name_input],
    upstreams=[copy_resource],
)

register_container_module = create_register_module(
    module_path=f"_automate.{SNAKE_APP_NAME_TPL}.container",
    alias=f"{SNAKE_APP_NAME_TPL}_container",
    inputs=[app_name_input],
    upstreams=[register_local_module],
)

register_deployment_module = create_register_module(
    module_path=f"_automate.{SNAKE_APP_NAME_TPL}.deployment",
    alias=f"{SNAKE_APP_NAME_TPL}_deployment",
    inputs=[app_name_input],
    upstreams=[register_container_module],
)

ensure_project_tasks = create_ensure_project_tasks(upstreams=[copy_resource])

add_start_upstream = create_add_start_upstream(
    upstream_module=f"_automate.{SNAKE_APP_NAME_TPL}.local",
    upstream_task_var=f"start_{SNAKE_APP_NAME_TPL}",
    upstreams=[ensure_project_tasks],
    inputs=[app_name_input],
)

add_start_container_upstream = create_add_start_containers_upstream(
    upstream_module=f"_automate.{SNAKE_APP_NAME_TPL}.container",
    upstream_task_var=f"start_{SNAKE_APP_NAME_TPL}_container",
    upstreams=[ensure_project_tasks],
    inputs=[app_name_input],
)

add_stop_container_upstream = create_add_stop_containers_upstream(
    upstream_module=f"_automate.{SNAKE_APP_NAME_TPL}.container",
    upstream_task_var=f"stop_{SNAKE_APP_NAME_TPL}_container",
    upstreams=[ensure_project_tasks],
    inputs=[app_name_input],
)

add_remove_container_upstream = create_add_remove_containers_upstream(
    upstream_module=f"_automate.{SNAKE_APP_NAME_TPL}.container",
    upstream_task_var=f"remove_{SNAKE_APP_NAME_TPL}_container",
    upstreams=[ensure_project_tasks],
    inputs=[app_name_input],
)

add_deploy_upstream = create_add_deploy_upstream(
    upstream_module=f"_automate.{SNAKE_APP_NAME_TPL}.deployment",
    upstream_task_var=f"deploy_{SNAKE_APP_NAME_TPL}",
    upstreams=[ensure_project_tasks],
    inputs=[app_name_input],
)

add_destroy_upstream = create_add_destroy_upstream(
    upstream_module=f"_automate.{SNAKE_APP_NAME_TPL}.deployment",
    upstream_task_var=f"destroy_{SNAKE_APP_NAME_TPL}",
    upstreams=[ensure_project_tasks],
    inputs=[app_name_input],
)


@python_task(
    name="kebab-zrb-meta-template-name",
    group=project_add_group,
    upstreams=[
        register_local_module,
        register_container_module,
        register_deployment_module,
        add_start_upstream,
        add_start_container_upstream,
        add_stop_container_upstream,
        add_remove_container_upstream,
        add_deploy_upstream,
        add_destroy_upstream,
    ],
    runner=runner,
)
async def snake_zrb_meta_template_name(*args: Any, **kwargs: Any):
    task: Task = kwargs.get("_task")
    task.print_out("Success")



from config import (
    app_auth_access_token_expire_seconds,
    app_auth_admin_active,
    app_auth_refresh_token_expire_seconds,
)
from integration.messagebus import publisher
from module.auth.entity.user.model import UserModel
from module.auth.integration import access_token_util, refresh_token_util
from module.auth.integration.model.permission_model import permission_model
from module.auth.integration.repo.user_repo import user_repo
from module.auth.integration.user import admin_user, admin_user_password, guest_user

user_model: UserModel = UserModel(
    repo=user_repo,
    publisher=publisher,
    permission_model=permission_model,
    access_token_util=access_token_util,
    access_token_expire_seconds=app_auth_access_token_expire_seconds,
    refresh_token_util=refresh_token_util,
    refresh_token_expire_seconds=app_auth_refresh_token_expire_seconds,
    guest_user=guest_user,
    admin_user=admin_user if app_auth_admin_active else None,
    admin_user_password=admin_user_password,
)



import base64

from zrb.builtin.group import base64_group
from zrb.helper.typing import Any
from zrb.runner import runner
from zrb.task.decorator import python_task
from zrb.task_input.str_input import StrInput

###############################################################################
# 🔤 Input Definitions
###############################################################################

text_input = StrInput(name="text", shortcut="t", description="Text", default="")

###############################################################################
# Task Definitions
###############################################################################


@python_task(
    name="encode",
    group=base64_group,
    inputs=[text_input],
    description="Encode a text using base64 algorithm",
    retry=0,
    runner=runner,
)
async def encode(*args: str, **kwargs: Any):
    text: str = kwargs.get("text", "")
    encoded_text = base64.b64encode(text.encode())
    return encoded_text.decode()


@python_task(
    name="decode",
    group=base64_group,
    inputs=[text_input],
    description="Decode a base64 encoded text",
    retry=0,
    runner=runner,
)
async def decode(*args: str, **kwargs: Any):
    text: str = kwargs.get("text", "")
    encoded_text = base64.b64decode(text.encode())
    return encoded_text.decode()



from integration.db_connection import engine
from integration.log import logger
from module.auth.entity.group.repo import GroupDBRepo, GroupRepo

group_repo: GroupRepo = GroupDBRepo(logger=logger, engine=engine)



import random
import string

from zrb.helper.typecheck import typechecked


@typechecked
def get_random_name(
    separator: str = "-", add_random_digit: bool = True, digit_count: int = 4
) -> str:
    prefixes = [
        "albedo",
        "argent",
        "argentum",
        "aurora",
        "aurum",
        "azure",
        "basilisk",
        "cerulean",
        "chimeric",
        "citrin",
        "coral",
        "crimson",
        "diamond",
        "draco",
        "dragon",
        "emerald",
        "ethereal",
        "ferrum",
        "flammeus",
        "garnet",
        "glacial",
        "glimmering",
        "glistening",
        "golden",
        "helios",
        "igneous",
        "imperial",
        "jade",
        "luminous",
        "luna",
        "lunar",
        "mystic",
        "nephrite",
        "nocturnal",
        "obsidian",
        "opal",
        "pearl",
        "platinum",
        "prismatic",
        "ruby",
        "sapphire",
        "serpentine",
        "silver",
        "sol",
        "solar",
        "spiritual",
        "stellar",
        "tempest",
        "topaz",
        "turquoise",
        "verde",
        "vermillion",
        "vitreous",
        "zephyr",
        "zircon",
    ]
    suffixes = [
        "aether",
        "albedo",
        "alchemy",
        "arcana",
        "aureum",
        "aetheris",
        "anima",
        "astralis",
        "caelestis",
        "chrysopoeia",
        "cosmicum",
        "crystallum",
        "deum",
        "divinitas",
        "draconis",
        "elementorum",
        "elixir",
        "essentia",
        "eternis",
        "ethereus",
        "fatum",
        "flamma",
        "fulgur",
        "hermetica",
        "ignis",
        "illuminationis",
        "imperium",
        "incantatum",
        "infinitum",
        "lapis",
        "lux",
        "magicae",
        "magnum",
        "materia",
        "metallum",
        "mysticum",
        "natura",
        "occultum",
        "omnipotentis",
        "opulentia",
        "philosophia",
        "philosophorum",
        "praeparatum",
        "praestantissimum",
        "prima",
        "primordium",
        "quintessentia",
        "regeneratio",
        "ritualis",
        "sanctum",
        "spiritus",
        "tenebris",
        "terra",
        "tinctura",
        "transmutationis",
        "universalis",
        "vapores",
        "venenum",
        "veritas",
        "vitae",
        "volatus",
    ]
    prefix = random.choice(prefixes)
    suffix = random.choice(suffixes)
    parts = [prefix, suffix]
    if add_random_digit:
        random_digit = "".join(random.choices(string.digits, k=digit_count))
        parts.append(random_digit)
    return separator.join(parts)



"""A Kubernetes Python Pulumi program to deploy kebab-zrb-app-name"""

import os

import pulumi
import pulumi_kubernetes as k8s
from dotenv import dotenv_values

CURRENT_DIR = os.path.dirname(__file__)
APP_DIR = os.path.abspath(os.path.join(CURRENT_DIR, "..", "src"))
TEMPLATE_ENV_FILE_NAME = os.path.join(APP_DIR, "template.env")

image = os.getenv("IMAGE", "kebab-zrb-app-name:latest")
replica = int(os.getenv("REPLICA", "1"))
app_labels = {"app": "kebab-zrb-app-name"}
env_map = dotenv_values(TEMPLATE_ENV_FILE_NAME)
app_port = int(os.getenv("APP_PORT", env_map.get("APP_PORT", "8080")))

# Pulumi deployment docs:
# https://www.pulumi.com/registry/packages/kubernetes/api-docs/apps/v1/deployment/
deployment = k8s.apps.v1.Deployment(
    resource_name="kebab-zrb-app-name",
    spec=k8s.apps.v1.DeploymentSpecArgs(
        selector=k8s.meta.v1.LabelSelectorArgs(match_labels=app_labels),
        replicas=replica,
        template=k8s.core.v1.PodTemplateSpecArgs(
            metadata=k8s.meta.v1.ObjectMetaArgs(labels=app_labels),
            spec=k8s.core.v1.PodSpecArgs(
                containers=[
                    k8s.core.v1.ContainerArgs(
                        name="kebab-zrb-app-name",
                        image=image,
                        env=[
                            k8s.core.v1.EnvVarArgs(
                                name=env_name, value=os.getenv(env_name, default_value)
                            )
                            for env_name, default_value in env_map.items()
                        ],
                        ports=[k8s.core.v1.ContainerPortArgs(container_port=app_port)],
                        liveness_probe=k8s.core.v1.ProbeArgs(
                            http_get=k8s.core.v1.HTTPGetActionArgs(port=app_port)
                        ),
                        readiness_probe=k8s.core.v1.ProbeArgs(
                            http_get=k8s.core.v1.HTTPGetActionArgs(port=app_port)
                        ),
                    )
                ]
            ),
        ),
    ),
)

# Pulumi services docs:
# https://www.pulumi.com/registry/packages/kubernetes/api-docs/core/v1/service/
service = k8s.core.v1.Service(
    resource_name="kebab-zrb-app-name",
    spec=k8s.core.v1.ServiceSpecArgs(
        selector=app_labels,
        ports=[
            k8s.core.v1.ServicePortArgs(
                port=app_port,
                protocol="TCP",
                target_port=app_port,
            )
        ],
        type="LoadBalancer",
    ),
)

pulumi.export("deployment-name", deployment.metadata["name"])
pulumi.export("service-name", service.metadata["name"])



import random

from termcolor import COLORS
from termcolor import colored as term_colored

from zrb.helper.typecheck import typechecked
from zrb.helper.typing import Iterable, Optional


@typechecked
def get_random_color() -> str:
    colors = [
        "green",
        "yellow",
        "blue",
        "magenta",
        "cyan",
        "white",
        "light_green",
        "light_yellow",
        "light_blue",
        "light_magenta",
        "light_cyan",
    ]
    return random.choice(colors)


@typechecked
def is_valid_color(color: str) -> bool:
    return color in COLORS


@typechecked
def colored(
    text: str,
    color: Optional[str] = None,
    on_color: Optional[str] = None,
    attrs: Optional[Iterable[str]] = None,
) -> str:
    return term_colored(text, color, on_color, attrs)



import os

from zrb.builtin.generator.common.helper import (
    register_module_to_project,
    validate_existing_project_dir,
)
from zrb.builtin.generator.common.task_input import project_dir_input
from zrb.helper.codemod.add_import_module import add_import_module
from zrb.helper.codemod.add_upstream_to_task import add_upstream_to_task
from zrb.helper.file.text import read_text_file_async, write_text_file_async
from zrb.helper.typecheck import typechecked
from zrb.helper.typing import Any, List, Optional
from zrb.task.any_task import AnyTask
from zrb.task.decorator import python_task
from zrb.task.task import Task
from zrb.task_input.any_input import AnyInput


@typechecked
def create_register_module(
    module_path: str,
    inputs: Optional[List[AnyInput]] = None,
    upstreams: Optional[List[AnyTask]] = None,
    alias: Optional[str] = None,
) -> Task:
    @python_task(
        name="register-module",
        inputs=[project_dir_input] + inputs if inputs is not None else [],
        upstreams=upstreams if upstreams is not None else [],
    )
    async def register_module(*args: Any, **kwargs: Any):
        task: Task = kwargs.get("_task")
        project_dir = kwargs.get("project_dir")
        validate_existing_project_dir(project_dir)
        rendered_module_path = task.render_str(module_path)
        rendered_alias: Optional[str] = None
        if alias is not None:
            rendered_alias = task.render_str(alias)
        task.print_out(
            f"Register module: {rendered_module_path}" + f" as {rendered_alias}"
            if rendered_alias is not None
            else ""
        )
        await register_module_to_project(
            project_dir=project_dir,
            module_path=rendered_module_path,
            alias=rendered_alias,
        )

    return register_module


@typechecked
def create_add_upstream(
    task_file_name: str,
    task_name: str,
    upstream_module: str,
    upstream_task_var: str,
    inputs: Optional[List[AnyInput]] = None,
    upstreams: Optional[List[AnyTask]] = None,
) -> Task:
    @python_task(
        name="register-upstream",
        inputs=[project_dir_input] + inputs if inputs is not None else [],
        upstreams=upstreams if upstreams is not None else [],
    )
    async def register_upstream(*args: Any, **kwargs: Any):
        task: Task = kwargs.get("_task")
        project_dir = kwargs.get("project_dir", ".")
        rendered_task_file_name = task.render_str(task_file_name)
        rendered_task_name = task.render_str(task_name)
        rendered_upstream_module_path = task.render_str(upstream_module)
        rendered_upstream_task_var = task.render_str(upstream_task_var)
        if not os.path.isabs(rendered_task_file_name):
            rendered_task_file_name = os.path.join(project_dir, rendered_task_file_name)
        code = await read_text_file_async(rendered_task_file_name)
        code = add_import_module(
            code=code,
            module_path=rendered_upstream_module_path,
            resource=rendered_upstream_task_var,
        )
        code = add_upstream_to_task(
            code, rendered_task_name, rendered_upstream_task_var
        )
        await write_text_file_async(rendered_task_file_name, code)

    return register_upstream



import json
import os
from typing import List

from helper.conversion import str_to_boolean, str_to_logging_level

app_src_dir = os.path.dirname(__file__)

zrb_app_name = os.getenv("APP_NAME", "app")
app_logging_level = str_to_logging_level(os.getenv("APP_LOGGING_LEVEL", "INFO"))
app_host = os.getenv("APP_HOST", "0.0.0.0")
app_port = int(os.getenv("APP_PORT", "8080"))
app_reload = str_to_boolean(os.getenv("APP_RELOAD", "true"))
app_max_not_ready = int(os.getenv("APP_MAX_NOT_READY", "10"))

app_enable_otel = str_to_boolean(os.getenv("APP_ENABLE_OTEL", "false"))
app_otel_exporter_otlp_endpoint = os.getenv("APP_OTEL_EXPORTER_OTLP_ENDPOINT", "false")

app_auth_access_token_cookie_key = os.getenv(
    "PUBLIC_AUTH_ACCESS_TOKEN_COOKIE_KEY", "access_token"
)
app_auth_refresh_token_cookie_key = os.getenv(
    "PUBLIC_AUTH_REFRESH_TOKEN_COOKIE_KEY", "refresh_token"
)

app_auth_access_token_type = os.getenv("APP_AUTH_ACCESS_TOKEN_TYPE", "jwt")
app_auth_access_token_expire_seconds = int(
    os.getenv("APP_AUTH_ACCESS_TOKEN_EXPIRE_SECONDS", "300")
)
app_auth_refresh_token_type = os.getenv("APP_AUTH_REFRESH_TOKEN_TYPE", "jwt")
app_auth_refresh_token_expire_seconds = int(
    os.getenv("APP_AUTH_REFRESH_TOKEN_EXPIRE_SECONDS", "86400")
)
app_auth_jwt_token_secret_key = os.getenv("APP_AUTH_JWT_TOKEN_SECRET_KEY", "secret")
app_auth_jwt_token_algorithm = os.getenv("APP_AUTH_JWT_TOKEN_ALGORITHM", "HS256")

app_auth_admin_active = str_to_boolean(os.getenv("APP_AUTH_ADMIN_ACTIVE", "true"))
app_auth_admin_user_id = os.getenv("APP_AUTH_ADMIN_USER_ID", "root")
app_auth_admin_username = os.getenv("APP_AUTH_ADMIN_USERNAME", "admin")
app_auth_admin_password = os.getenv("APP_AUTH_ADMIN_PASSWORD", "admin")
app_auth_admin_email = os.getenv("APP_AUTH_ADMIN_EMAIL", "")
app_auth_admin_phone = os.getenv("APP_AUTH_ADMIN_PHONE", "")
app_auth_guest_user_id = os.getenv("APP_AUTH_GUEST_USER_ID", "guest")
app_auth_guest_username = os.getenv("APP_AUTH_GUEST_USERNAME", "guest")
app_auth_guest_email = os.getenv("APP_AUTH_GUEST_EMAIL", "")
app_auth_guest_phone = os.getenv("APP_AUTH_GUEST_PHONE", "")

app_db_connection = os.getenv("APP_DB_CONNECTION", "sqlite://")
app_db_engine_show_log = str_to_boolean(os.getenv("APP_DB_ENGINE_SHOW_LOG", "false"))
app_db_auto_migrate = str_to_boolean(os.getenv("APP_DB_AUTO_MIGRATE", "true"))

app_broker_type = os.getenv("APP_BROKER_TYPE", "mock")

app_enable_rpc_server: bool = str_to_boolean(os.getenv("APP_ENABLE_RPC_SERVER", "true"))

app_enable_event_handler: bool = str_to_boolean(
    os.getenv("APP_ENABLE_EVENT_HANDLER", "true")
)

app_enable_frontend: bool = str_to_boolean(os.getenv("APP_ENABLE_FRONTEND", "true"))

app_enable_api: bool = str_to_boolean(os.getenv("APP_ENABLE_API", "true"))

app_rmq_connection_string = os.getenv(
    "APP_RMQ_CONNECTION", "amqp://guest:guest@localhost/"
)

app_kafka_bootstrap_servers = os.getenv("APP_KAFKA_BOOTSTRAP_SERVERS", "localhost:9092")
app_kafka_security_protocol = os.getenv("APP_KAFKA_SECURITY_PROTOCOL", "PLAINTEXT")
app_kafka_sasl_mechanism = os.getenv("APP_KAFKA_SASL_MECHANISM", "SCRAM-SHA-512")
app_kafka_sasl_user = os.getenv("APP_KAFKA_SASL_USER", "admin")
app_kafka_sasl_pass = os.getenv("APP_KAFKA_SASL_PASS", "admin")

app_cors_allow_origins: List[str] = json.loads(
    os.getenv("APP_CORS_ALLOW_ORIGINS", '["*"]')
)
app_cors_allow_origin_regex: str = os.getenv("APP_CORS_ALLOW_ORIGIN_REGEX", "")
app_cors_allow_methods: List[str] = json.loads(
    os.getenv("APP_CORS_ALLOW_METHODS", '["*"]')
)
app_cors_allow_headers: List[str] = json.loads(
    os.getenv("APP_CORS_ALLOW_HEADERS", '["*"]')
)
app_cors_allow_credentials: bool = str_to_boolean(
    os.getenv("APP_CORS_ALLOW_CREDENTIALS", "false")
)
app_cors_expose_headers: bool = str_to_boolean(
    os.getenv("APP_CORS_EXPOSE_HEADERS", "false")
)
app_cors_max_age: int = int(os.getenv("APP_CORS_MAX_AGE", "600"))
app_enable_auth_module = str_to_boolean(os.getenv("APP_ENABLE_AUTH_MODULE", "true"))
app_enable_log_module = str_to_boolean(os.getenv("APP_ENABLE_LOG_MODULE", "true"))

public_brand = os.getenv("PUBLIC_BRAND", "PascalZrbAppName")
public_title = os.getenv("PUBLIC_TITLE", "PascalZrbAppName")
public_auth_access_token_cookie_key = os.getenv(
    "PUBLIC_AUTH_ACCESS_TOKEN_COOKIE_KEY", "access_token"
)
public_auth_refresh_token_cookie_key = os.getenv(
    "PUBLIC_AUTH_REFRESH_TOKEN_COOKIE_KEY", "refresh_token"
)



import os

from zrb.builtin.generator.common.helper import (
    validate_existing_project_dir,
    validate_inexisting_automation,
)
from zrb.builtin.generator.common.task_factory import create_register_module
from zrb.builtin.generator.common.task_input import project_dir_input
from zrb.builtin.group import project_add_group
from zrb.helper.accessories.name import get_random_name
from zrb.helper.typing import Any
from zrb.runner import runner
from zrb.task.cmd_task import CmdTask
from zrb.task.decorator import python_task
from zrb.task.resource_maker import ResourceMaker
from zrb.task.task import Task
from zrb.task_input.bool_input import BoolInput
from zrb.task_input.int_input import IntInput
from zrb.task_input.str_input import StrInput

CURRENT_DIR = os.path.dirname(__file__)

###############################################################################
# Task Input Definitions
###############################################################################

template_name_input = StrInput(
    name="template-name",
    shortcut="t",
    description="Template name",
    prompt="Template name",
    default=get_random_name(),
)

base_image_input = StrInput(
    name="base-image",
    shortcut="i",
    description="Base image",
    prompt="Base image",
    default="stalchmst/moku:1.0.0",
)

default_app_port_input = IntInput(
    name="default-app-port",
    shortcut="p",
    description="Default app port",
    prompt="Default app port",
    default="8080",
)

build_custom_image_input = BoolInput(
    name="build-custom-image",
    description="Whether build custom image or not",
    prompt="Does user need to create custom image?",
    default=True,
)

is_container_only_input = BoolInput(
    name="is-container-only",
    description="Whether app only run as container or not",
    prompt="Is this container only?",
    default=False,
)

is_http_port_input = BoolInput(
    name="is-http-port",
    description="Whether app run on top of HTTP(s) protocol or not",
    prompt="Is this a web appp (run on top of HTTP(s))?",
    default=False,
)

use_helm_input = BoolInput(
    name="use-helm",
    description="Whether using helm for deployment or not",
    prompt="Do you want to use helm for deployment?",
    default=False,
)

###############################################################################
# Task Properties
###############################################################################

inputs = [
    project_dir_input,
    template_name_input,
    default_app_port_input,
    is_http_port_input,
    base_image_input,
    build_custom_image_input,
    is_container_only_input,
    use_helm_input,
]

replacements = {
    "zrbMetaTemplateName": "{{input.template_name}}",
    "zrbMetaBaseImage": "{{input.base_image}}",
    "zrbMetaDefaultAppPort": "{{input.default_app_port}}",
}

###############################################################################
# Task Definitions
###############################################################################


@python_task(name="validate", inputs=inputs, retry=0)
async def validate(*args: Any, **kwargs: Any):
    project_dir = kwargs.get("project_dir")
    validate_existing_project_dir(project_dir)
    template_name = kwargs.get("template_name")
    validate_inexisting_automation(project_dir, template_name)
    is_container_only_input: bool = kwargs.get("is_container_only")
    build_custom_image_input: bool = kwargs.get("build_custom_image")
    if not is_container_only_input and not build_custom_image_input:
        raise Exception(
            "Invalid options: Not is-container-only but not build-custom-image"
        )


copy_base_resource = ResourceMaker(
    name="copy-base-resource",
    inputs=inputs,
    upstreams=[validate],
    replacements=replacements,
    template_path=os.path.join(CURRENT_DIR, "template", "base"),
    destination_path="{{ input.project_dir }}",
    excludes=["*/__pycache__"],
)

copy_http_port_resource = ResourceMaker(
    name="copy-http-port-resource",
    inputs=inputs,
    should_execute="{{ input.is_http_port }}",
    upstreams=[copy_base_resource],
    replacements=replacements,
    template_path=os.path.join(CURRENT_DIR, "template", "http-port"),
    destination_path="{{ input.project_dir }}",
    excludes=["*/__pycache__"],
)

copy_custom_image_resource = ResourceMaker(
    name="copy-custom-image-resource",
    inputs=inputs,
    should_execute="{{ input.build_custom_image }}",
    upstreams=[copy_http_port_resource],
    replacements=replacements,
    template_path=os.path.join(CURRENT_DIR, "template", "build-custom-image"),
    destination_path="{{ input.project_dir }}",
    excludes=["*/__pycache__"],
)

copy_http_port_custom_image_resource = ResourceMaker(
    name="copy-http-port-custom-image-resource",
    inputs=inputs,
    should_execute="{{ input.is_http_port and input.build_custom_image }}",
    upstreams=[copy_custom_image_resource],
    replacements=replacements,
    template_path=os.path.join(
        CURRENT_DIR, "template", "http-port-build-custom-image"
    ),  # noqa
    destination_path="{{ input.project_dir }}",
    excludes=["*/__pycache__"],
)

copy_helm_resource = ResourceMaker(
    name="copy-helm-resource",
    inputs=inputs,
    should_execute="{{ input.use_helm }}",
    upstreams=[copy_http_port_custom_image_resource],
    replacements=replacements,
    template_path=os.path.join(CURRENT_DIR, "template", "use-helm"),
    destination_path="{{ input.project_dir }}",
    excludes=["*/__pycache__"],
)

copy_resource = CmdTask(
    name="copy-resource",
    upstreams=[
        copy_helm_resource,
    ],
    cmd="echo Resource copied",
)

register_module = create_register_module(
    module_path="_automate.generate_{{util.to_snake_case(input.template_name)}}.add",  # noqa
    alias="generate_{{util.to_snake_case(input.template_name)}}",
    inputs=[template_name_input],
    upstreams=[copy_resource],
)


@python_task(
    name="app-generator",
    group=project_add_group,
    upstreams=[register_module],
    inputs=inputs,
    runner=runner,
)
async def add_app_generator(*args: Any, **kwargs: Any):
    task: Task = kwargs.get("_task")
    task.print_out("Success")



from module.log.component.historical_repo_model import HistoricalRepoModel
from module.snake_zrb_module_name.schema.snake_zrb_entity_name import (
    PascalZrbEntityName,
    PascalZrbEntityNameData,
    PascalZrbEntityNameResult,
)


class PascalZrbEntityNameModel(
    HistoricalRepoModel[
        PascalZrbEntityName, PascalZrbEntityNameData, PascalZrbEntityNameResult
    ]
):
    schema_result_cls = PascalZrbEntityNameResult
    log_entity_name = "snake_zrb_entity_name"



from zrb.task_group.group import Group

project_group = Group(name="project", description="Project management")
project_add_group = Group(
    name="add", description="Add resources to project", parent=project_group
)
env_group = Group(name="env", description="Environment variable management")
ubuntu_group = Group(name="ubuntu", description="Ubuntu related commands")
ubuntu_install_group = Group(
    name="install", description="Install things on ubuntu", parent=ubuntu_group
)
explain_group = Group(name="explain", description="Explain things")
base64_group = Group(name="base64", description="Base64 operations")
md5_group = Group(name="md5", description="MD5 operations")
dev_tool_group = Group(name="devtool", description="Developer tools management")
dev_tool_install_group = Group(
    name="install", description="Install developer tools", parent=dev_tool_group
)
git_group = Group(name="git", description="Git related commands")
docker_group = Group(name="docker", description="Docker related commands")
process_group = Group(name="process", description="Process related commands")
plugin_group = Group(name="plugin", description="Plugin related command")



from logging import Logger

from component.messagebus import Publisher
from component.rpc import Caller
from fastapi import FastAPI
from module.auth.component import Authorizer


def register_api(
    logger: Logger,
    app: FastAPI,
    authorizer: Authorizer,
    rpc_caller: Caller,
    publisher: Publisher,
):
    logger.info('🥪 Register API for "snake_zrb_module_name"')



import os
import re

from zrb.helper.codemod.add_key_value_to_dict import add_key_value_to_dict
from zrb.helper.codemod.add_property_to_class import add_property_to_class
from zrb.helper.file.text import read_text_file_async, write_text_file_async
from zrb.helper.typecheck import typechecked
from zrb.task.task import Task


@typechecked
async def add_column_to_insert_page(
    task: Task,
    project_dir: str,
    kebab_app_name: str,
    kebab_module_name: str,
    kebab_entity_name: str,
    kebab_column_name: str,
    snake_column_name: str,
    capitalized_human_readable_column_name: str,
):
    list_page_file_path = os.path.join(
        project_dir,
        "src",
        kebab_app_name,
        "src",
        "frontend",
        "src",
        "routes",
        kebab_module_name,
        kebab_entity_name,
        "new",
        "+page.svelte",
    )
    task.print_out(f"Read HTML from: {list_page_file_path}")
    html_content = await read_text_file_async(list_page_file_path)
    task.print_out("Add default value to insert page")
    default_value_regex = r"(.*)(// DON'T DELETE: set field default value here)"  # noqa
    default_value_subst = "\\n".join([f"\\1row.{snake_column_name} = '';", "\\1\\2"])
    html_content = re.sub(
        default_value_regex, default_value_subst, html_content, 0, re.MULTILINE
    )
    task.print_out("Add field to insert page")
    field_regex = r"(.*)(<!-- DON'T DELETE: insert new field here-->)"
    field_subst = "\\n".join(
        [
            '\\1<div class="mb-4">',
            f'\\1    <label class="block text-gray-700 font-bold mb-2" for="{kebab_column_name}">{capitalized_human_readable_column_name}</label>',  # noqa
            f'\\1    <input type="text" class="input w-full" id="{kebab_column_name}" placeholder="{capitalized_human_readable_column_name}" bind:value='
            + "{row."
            + snake_column_name
            + "} />",  # noqa
            "\\1</div>",
            "\\1\\2",
        ]
    )
    html_content = re.sub(field_regex, field_subst, html_content, 0, re.MULTILINE)
    task.print_out(f"Write modified HTML to: {list_page_file_path}")
    await write_text_file_async(list_page_file_path, html_content)


@typechecked
async def add_column_to_update_page(
    task: Task,
    project_dir: str,
    kebab_app_name: str,
    kebab_module_name: str,
    kebab_entity_name: str,
    kebab_column_name: str,
    snake_column_name: str,
    capitalized_human_readable_column_name: str,
):
    list_page_file_path = os.path.join(
        project_dir,
        "src",
        kebab_app_name,
        "src",
        "frontend",
        "src",
        "routes",
        kebab_module_name,
        kebab_entity_name,
        "update",
        "[id]",
        "+page.svelte",
    )
    task.print_out(f"Read HTML from: {list_page_file_path}")
    html_content = await read_text_file_async(list_page_file_path)
    task.print_out("Add field to update page")
    regex = r"(.*)(<!-- DON'T DELETE: insert new field here-->)"
    subst = "\\n".join(
        [
            '\\1<div class="mb-4">',
            f'\\1    <label class="block text-gray-700 font-bold mb-2" for="{kebab_column_name}">{capitalized_human_readable_column_name}</label>',  # noqa
            f'\\1    <input type="text" class="input w-full" id="{kebab_column_name}" placeholder="{capitalized_human_readable_column_name}" bind:value='
            + "{row."
            + snake_column_name
            + "} />",  # noqa
            "\\1</div>",
            "\\1\\2",
        ]
    )
    html_content = re.sub(regex, subst, html_content, 0, re.MULTILINE)
    task.print_out(f"Write modified HTML to: {list_page_file_path}")
    await write_text_file_async(list_page_file_path, html_content)


@typechecked
async def add_column_to_delete_page(
    task: Task,
    project_dir: str,
    kebab_app_name: str,
    kebab_module_name: str,
    kebab_entity_name: str,
    kebab_column_name: str,
    snake_column_name: str,
    capitalized_human_readable_column_name: str,
):
    list_page_file_path = os.path.join(
        project_dir,
        "src",
        kebab_app_name,
        "src",
        "frontend",
        "src",
        "routes",
        kebab_module_name,
        kebab_entity_name,
        "delete",
        "[id]",
        "+page.svelte",
    )
    task.print_out(f"Read HTML from: {list_page_file_path}")
    html_content = await read_text_file_async(list_page_file_path)
    task.print_out("Add field to delete page")
    regex = r"(.*)(<!-- DON'T DELETE: insert new field here-->)"
    subst = "\\n".join(
        [
            '\\1<div class="mb-4">',
            f'\\1    <label class="block text-gray-700 font-bold mb-2" for="{kebab_column_name}">{capitalized_human_readable_column_name}</label>',  # noqa
            f'\\1    <span id="{kebab_column_name}">'
            + "{row."
            + snake_column_name
            + "}</span>",  # noqa
            "\\1</div>",
            "\\1\\2",
        ]
    )
    html_content = re.sub(regex, subst, html_content, 0, re.MULTILINE)
    task.print_out(f"Write modified HTML to: {list_page_file_path}")
    await write_text_file_async(list_page_file_path, html_content)


@typechecked
async def add_column_to_detail_page(
    task: Task,
    project_dir: str,
    kebab_app_name: str,
    kebab_module_name: str,
    kebab_entity_name: str,
    kebab_column_name: str,
    snake_column_name: str,
    capitalized_human_readable_column_name: str,
):
    list_page_file_path = os.path.join(
        project_dir,
        "src",
        kebab_app_name,
        "src",
        "frontend",
        "src",
        "routes",
        kebab_module_name,
        kebab_entity_name,
        "detail",
        "[id]",
        "+page.svelte",
    )
    task.print_out(f"Read HTML from: {list_page_file_path}")
    html_content = await read_text_file_async(list_page_file_path)
    task.print_out("Add field to detail page")
    regex = r"(.*)(<!-- DON'T DELETE: insert new field here-->)"
    subst = "\\n".join(
        [
            '\\1<div class="mb-4">',
            f'\\1    <label class="block text-gray-700 font-bold mb-2" for="{kebab_column_name}">{capitalized_human_readable_column_name}</label>',  # noqa
            f'\\1    <span id="{kebab_column_name}">'
            + "{row."
            + snake_column_name
            + "}</span>",  # noqa
            "\\1</div>",
            "\\1\\2",
        ]
    )
    html_content = re.sub(regex, subst, html_content, 0, re.MULTILINE)
    task.print_out(f"Write modified HTML to: {list_page_file_path}")
    await write_text_file_async(list_page_file_path, html_content)


@typechecked
async def add_column_to_list_page(
    task: Task,
    project_dir: str,
    kebab_app_name: str,
    kebab_module_name: str,
    kebab_entity_name: str,
    snake_column_name: str,
    capitalized_human_readable_column_name: str,
):
    list_page_file_path = os.path.join(
        project_dir,
        "src",
        kebab_app_name,
        "src",
        "frontend",
        "src",
        "routes",
        kebab_module_name,
        kebab_entity_name,
        "+page.svelte",
    )
    task.print_out(f"Read HTML from: {list_page_file_path}")
    html_content = await read_text_file_async(list_page_file_path)
    # process header
    task.print_out("Add column header to table")
    header_regex = r"(.*)(<!-- DON'T DELETE: insert new column header here-->)"
    header_subst = "\\n".join(
        [f"\\1<th>{capitalized_human_readable_column_name}</th>", "\\1\\2"]
    )
    html_content = re.sub(header_regex, header_subst, html_content, 0, re.MULTILINE)
    # process column
    task.print_out("Add column to table")
    column_regex = r"(.*)(<!-- DON'T DELETE: insert new column here-->)"
    column_subst = "\\n".join(["\\1<td>{row." + snake_column_name + "}</td>", "\\1\\2"])
    html_content = re.sub(column_regex, column_subst, html_content, 0, re.MULTILINE)
    task.print_out(f"Write modified HTML to: {list_page_file_path}")
    await write_text_file_async(list_page_file_path, html_content)


@typechecked
async def add_column_to_test(
    task: Task,
    project_dir: str,
    kebab_app_name: str,
    snake_module_name: str,
    snake_entity_name: str,
    snake_column_name: str,
    column_type: str,
):
    test_file_path = os.path.join(
        project_dir,
        "src",
        kebab_app_name,
        "test",
        snake_module_name,
        f"test_{snake_entity_name}.py",
    )
    task.print_out(f"Read code from: {test_file_path}")
    code = await read_text_file_async(test_file_path)
    task.print_out(f'Add column "{snake_column_name}" to the test')
    dict_names = [
        "inserted_success_data",
        "to_be_updated_success_data",
        "updated_success_data",
        "to_be_deleted_success_data",
    ]
    for dict_name in dict_names:
        code = add_key_value_to_dict(
            code=code, dict_name=dict_name, key=f"'{snake_column_name}'", value="''"
        )
    task.print_out(f"Write modified code to: {test_file_path}")
    await write_text_file_async(test_file_path, code)


@typechecked
async def add_column_to_schema(
    task: Task,
    project_dir: str,
    kebab_app_name: str,
    snake_module_name: str,
    snake_entity_name: str,
    pascal_entity_name: str,
    snake_column_name: str,
    column_type: str,
):
    schema_file_path = os.path.join(
        project_dir,
        "src",
        kebab_app_name,
        "src",
        "module",
        snake_module_name,
        "schema",
        f"{snake_entity_name}.py",
    )
    task.print_out(f"Read code from: {schema_file_path}")
    code = await read_text_file_async(schema_file_path)
    task.print_out(f'Add column "{snake_column_name}" to the schema')
    code = add_property_to_class(
        code=code,
        class_name=f"{pascal_entity_name}Data",
        property_name=snake_column_name,
        property_type="str",
    )
    task.print_out(f"Write modified code to: {schema_file_path}")
    await write_text_file_async(schema_file_path, code)


@typechecked
async def add_column_to_repo(
    task: Task,
    project_dir: str,
    kebab_app_name: str,
    snake_module_name: str,
    snake_entity_name: str,
    pascal_entity_name: str,
    snake_column_name: str,
    column_type: str,
):
    repo_file_path = os.path.join(
        project_dir,
        "src",
        kebab_app_name,
        "src",
        "module",
        snake_module_name,
        "entity",
        snake_entity_name,
        "repo.py",
    )
    task.print_out(f"Read code from: {repo_file_path}")
    code = await read_text_file_async(repo_file_path)
    task.print_out(f'Add column "{snake_column_name}" to the repo')
    code = add_property_to_class(
        code=code,
        class_name=f"DBEntity{pascal_entity_name}",
        property_name=snake_column_name,
        property_type="Column",
        property_value="Column(String)",
    )
    task.print_out(f"Write modified code to: {repo_file_path}")
    await write_text_file_async(repo_file_path, code)



from zrb.helper.typecheck import typechecked
from zrb.helper.typing import Any, Optional, Union
from zrb.task_input.base_input import BaseInput

# flake8: noqa E501


@typechecked
class FloatInput(BaseInput):
    """
     A specialized input class designed to handle floating-point numbers in a task input context.

    Extending `BaseInput`, this class facilitates the handling of float-type inputs, accommodating
    various features like default values, prompts, flags, and customization options.

    Args:
        name (str): The name of the input.
        shortcut (Optional[str]): An optional shortcut string for the input.
        default (Optional[Any]): The default value for the input, expected to be a float if set.
        description (Optional[str]): A brief description of the input's purpose.
        show_default (Union[bool, str, None]): Option to display the default value. Can be a boolean or string.
        prompt (Union[bool, str]): A boolean or string to prompt the user for input. If `True`, uses the default prompt.
        confirmation_prompt (Union[bool, str]): If `True`, the user is asked to confirm the input.
        prompt_required (bool): If `True`, the prompt for input is mandatory.
        hide_input (bool): If `True`, the input is hidden, useful for sensitive information.
        is_flag (Optional[bool]): Indicates if the input is a flag. If `True`, the input accepts boolean flag values.
        flag_value (Optional[Any]): The value associated with the flag if `is_flag` is `True`.
        multiple (bool): If `True`, allows multiple float values for the input.
        count (bool): If `True`, counts the occurrences of the input.
        allow_from_autoenv (bool): If `True`, allows the input to be automatically populated from the environment.
        hidden (bool): If `True`, the input is not shown in help messages or documentation.
        show_choices (bool): If `True`, displays the available choices to the user (relevant if there are restricted float choices).
        show_envvar (bool): Indicates whether to display the environment variable associated with this input.
        nargs (int): The number of arguments that the input can accept.
        should_render (bool): If `True`, the input is rendered in the UI or command-line interface.

    Examples:
        >>> float_input = FloatInput(name='threshold', default=0.5, description='Set the threshold value')
        >>> float_input.get_default()
        0.5
    """

    def __init__(
        self,
        name: str,
        shortcut: Optional[str] = None,
        default: Optional[Any] = None,
        description: Optional[str] = None,
        show_default: Union[bool, str, None] = None,
        prompt: Union[bool, str] = True,
        confirmation_prompt: Union[bool, str] = False,
        prompt_required: bool = True,
        hide_input: bool = False,
        is_flag: Optional[bool] = None,
        flag_value: Optional[Any] = None,
        multiple: bool = False,
        count: bool = False,
        allow_from_autoenv: bool = True,
        hidden: bool = False,
        show_choices: bool = True,
        show_envvar: bool = False,
        nargs: int = 1,
        should_render: bool = True,
    ):
        BaseInput.__init__(
            self,
            name=name,
            shortcut=shortcut,
            default=default,
            description=description,
            show_default=show_default,
            prompt=prompt,
            confirmation_prompt=confirmation_prompt,
            prompt_required=prompt_required,
            hide_input=hide_input,
            is_flag=is_flag,
            flag_value=flag_value,
            multiple=multiple,
            count=count,
            allow_from_autoenv=allow_from_autoenv,
            type=float,
            hidden=hidden,
            show_choices=show_choices,
            show_envvar=show_envvar,
            nargs=nargs,
            should_render=should_render,
        )



import os

from zrb.helper.accessories.name import get_random_name
from zrb.task_input.int_input import IntInput
from zrb.task_input.str_input import StrInput

SYSTEM_USER = os.getenv("USER", "incognito")

project_dir_input = StrInput(
    name="project-dir",
    shortcut="d",
    description="Project directory",
    prompt="Project directory",
    default=".",
)

project_name_input = StrInput(
    name="project-name",
    shortcut="n",
    description="Project name",
    prompt="Project name (can be empty)",
    default="",
)

project_author_name_input = StrInput(
    name="project-author-name",
    prompt="Project author name",
    description="Project author name",
    default=SYSTEM_USER,
)

project_author_email_input = StrInput(
    name="project-author-email",
    prompt="Project author email",
    description="Project author email",
    default=f"{SYSTEM_USER}@gmail.com",
)

project_description_input = StrInput(
    name="project-description",
    description="Project description",
    prompt="Project description",
    default="Just another Zrb project",
)

app_name_input = StrInput(
    name="app-name",
    shortcut="a",
    description="App name",
    prompt="App name",
    default="app",
)

app_author_name_input = StrInput(
    name="app-author-name",
    prompt="App author name",
    description="App author name",
    default=SYSTEM_USER,
)

app_author_email_input = StrInput(
    name="app-author-email",
    prompt="App author email",
    description="App author email",
    default=f"{SYSTEM_USER}@gmail.com",
)

app_description_input = StrInput(
    name="app-description",
    description="App description",
    prompt="App description",
    default="Just another app",
)

app_image_default_namespace = os.getenv(
    "PROJECT_IMAGE_DEFAULT_NAMESPACE", "docker.io/" + SYSTEM_USER
)
app_image_name_input = StrInput(
    name="app-image-name",
    description="App image name",
    prompt="App image name",
    default=app_image_default_namespace
    + "/"
    + "{{util.to_kebab_case(input.app_name)}}",  # noqa
)

module_name_input = StrInput(
    name="module-name",
    shortcut="m",
    description="module name",
    prompt="module name",
    default="library",
)

entity_name_input = StrInput(
    name="entity-name",
    shortcut="e",
    description="Singular entity name",
    prompt="Singular entity name",
    default="book",
)

plural_entity_name_input = StrInput(
    name="plural-entity-name",
    description="Plural entity name",
    prompt="Plural entity name",
    default="books",
)

main_column_name_input = StrInput(
    name="column-name",
    shortcut="c",
    description="Main column name",
    prompt="Main column name",
    default="code",
)

column_name_input = StrInput(
    name="column-name",
    shortcut="c",
    description="Column name",
    prompt="Column name",
    default="title",
)

column_type_input = StrInput(
    name="column-type",
    shortcut="t",
    description="Column type",
    prompt="Column type",
    default="str",
)

task_name_input = StrInput(
    name="task-name",
    shortcut="t",
    description="Task name",
    prompt="Task name",
    default=f"run-{get_random_name()}",
)

http_port_input = IntInput(
    name="http-port",
    shortcut="p",
    description="HTTP port",
    prompt="HTTP port",
    default=8080,
)

env_prefix_input = StrInput(
    name="env-prefix",
    description="OS environment prefix",
    prompt="OS environment prefix",
    default='{{util.to_snake_case(util.coalesce(input.app_name, input.task_name, "MY")).upper()}}',  # noqa
)

package_name_input = StrInput(
    name="package-name",
    shortcut="p",
    description="Package name",
    prompt="Package name",
    default=get_random_name(),
)

package_description_input = StrInput(
    name="package-description",
    description="Package description",
    prompt="Package description",
    default="Just another package",
)

package_homepage_input = StrInput(
    name="package-homepage",
    description="Package homepage",
    prompt="Package homepage",
    default="".join(
        [
            "https://github.com/",
            SYSTEM_USER,
            "/{{util.to_kebab_case(input.package_name)}}",
        ]
    ),
)

package_repository_input = StrInput(
    name="package-repository",
    description="Package repository",
    prompt="Package homepage",
    default="".join(
        [
            "https://github.com/",
            SYSTEM_USER,
            "/{{util.to_kebab_case(input.package_name)}}",
        ]
    ),
)

package_documentation_input = StrInput(
    name="package-documentation",
    description="Package documentation",
    prompt="Package homepage",
    default="".join(
        [
            "https://github.com/",
            SYSTEM_USER,
            "/{{util.to_kebab_case(input.package_name)}}",
        ]
    ),
)

package_author_name_input = StrInput(
    name="package-author-name",
    prompt="Package author name",
    description="Package author name",
    default=SYSTEM_USER,
)

package_author_email_input = StrInput(
    name="package-author-email",
    prompt="Package author email",
    description="Package author email",
    default=f"{SYSTEM_USER}@gmail.com",
)



import os

from zrb import CmdTask, runner
from zrb.builtin.group import project_group

from ._config import CURRENT_DIR, DEPLOYMENT_DIR
from ._env import (
    deployment_enable_monitoring_env,
    deployment_mode_env,
    deployment_modules_env,
    image_env,
    pulumi_backend_url_env,
    pulumi_config_passphrase_env,
)
from ._env_file import deployment_app_env_file, deployment_config_env_file
from ._input import (
    deploy_mode_input,
    enable_monitoring_input,
    image_input,
    pulumi_stack_input,
)
from .image import push_snake_zrb_app_name_image

###############################################################################
# ⚙️ deploy-kebab-zrb-task-name
###############################################################################

deploy_snake_zrb_app_name = CmdTask(
    icon="🚧",
    name="deploy-kebab-zrb-app-name",
    description="Deploy human readable zrb app name",
    group=project_group,
    inputs=[
        image_input,
        pulumi_stack_input,
        deploy_mode_input,
        enable_monitoring_input,
    ],
    upstreams=[push_snake_zrb_app_name_image],
    cwd=DEPLOYMENT_DIR,
    env_files=[
        deployment_config_env_file,
        deployment_app_env_file,
    ],
    envs=[
        pulumi_backend_url_env,
        pulumi_config_passphrase_env,
        image_env,
        deployment_modules_env,
        deployment_mode_env,
        deployment_enable_monitoring_env,
    ],
    cmd_path=[
        os.path.join(CURRENT_DIR, "cmd", "pulumi-init-stack.sh"),
        os.path.join(CURRENT_DIR, "cmd", "pulumi-up.sh"),
    ],
)
runner.register(deploy_snake_zrb_app_name)

###############################################################################
# ⚙️ destroy-kebab-zrb-task-name
###############################################################################

destroy_snake_zrb_app_name = CmdTask(
    icon="💨",
    name="destroy-kebab-zrb-app-name",
    description="Remove human readable zrb app name deployment",
    group=project_group,
    inputs=[
        pulumi_stack_input,
    ],
    cwd=DEPLOYMENT_DIR,
    env_files=[
        deployment_config_env_file,
        deployment_app_env_file,
    ],
    envs=[
        pulumi_backend_url_env,
        pulumi_config_passphrase_env,
        image_env,
        deployment_modules_env,
    ],
    cmd_path=[
        os.path.join(CURRENT_DIR, "cmd", "pulumi-init-stack.sh"),
        os.path.join(CURRENT_DIR, "cmd", "pulumi-destroy.sh"),
    ],
)
runner.register(destroy_snake_zrb_app_name)



from config import (
    app_auth_admin_email,
    app_auth_admin_password,
    app_auth_admin_phone,
    app_auth_admin_user_id,
    app_auth_admin_username,
    app_auth_guest_email,
    app_auth_guest_phone,
    app_auth_guest_user_id,
    app_auth_guest_username,
)
from module.auth.schema.user import User

admin_user_password = app_auth_admin_password

admin_user = User(
    id=app_auth_admin_user_id,
    username=app_auth_admin_username,
    email=app_auth_admin_email,
    phone=app_auth_admin_phone,
    groups=[],
    permissions=[],
    description="System administrator",
)

guest_user = User(
    id=app_auth_guest_user_id,
    username=app_auth_guest_username,
    email=app_auth_guest_email,
    phone=app_auth_guest_phone,
    groups=[],
    permissions=[],
    description="Visitor",
)



from module.auth.schema.group import Group, GroupData, GroupResult
from module.log.component.historical_repo_model import HistoricalRepoModel


class GroupModel(HistoricalRepoModel[Group, GroupData, GroupResult]):
    schema_result_cls = GroupResult
    log_entity_name = "group"



import asyncio
import atexit
import os
import pathlib
import signal
import subprocess
import sys
import time

from zrb.config.config import default_shell
from zrb.helper.string.conversion import to_variable_name
from zrb.helper.typecheck import typechecked
from zrb.helper.typing import (
    Any,
    Callable,
    Iterable,
    JinjaTemplate,
    List,
    Optional,
    TypeVar,
    Union,
)
from zrb.task.any_task import AnyTask
from zrb.task.any_task_event_handler import (
    OnFailed,
    OnReady,
    OnRetry,
    OnSkipped,
    OnStarted,
    OnTriggered,
    OnWaiting,
)
from zrb.task.base_task.base_task import BaseTask
from zrb.task_env.env import Env
from zrb.task_env.env_file import EnvFile
from zrb.task_group.group import Group
from zrb.task_input.any_input import AnyInput

_has_stty = True
try:
    _original_stty = subprocess.getoutput("stty -g").rstrip()
except Exception:
    _has_stty = False


def _reset_stty():
    global _has_stty
    if _has_stty:
        try:
            subprocess.run(["stty", _original_stty], check=True)
        except Exception:
            _has_stty = False


CmdVal = Union[
    JinjaTemplate,
    Iterable[JinjaTemplate],
    Callable[..., Union[Iterable[JinjaTemplate], JinjaTemplate]],
]
TCmdTask = TypeVar("TCmdTask", bound="CmdTask")


class CmdResult:
    def __init__(self, output: str, error: str):
        self.output = output
        self.error = error

    def __str__(self) -> str:
        return self.output


class CmdGlobalState:
    def __init__(self):
        self.no_more_attempt: bool = False
        self.is_killed_by_signal: bool = False


@typechecked
class CmdTask(BaseTask):
    """
    Command Task.
    You can use this task to run shell command.

    Examples:
        >>> from zrb import runner, CmdTask, StrInput, Env
        >>> hello = CmdTask(
        >>>     name='hello',
        >>>     inputs=[StrInput(name='name', default='World')],
        >>>     envs=[Env(name='HOME_DIR', os_name='HOME')],
        >>>     cmd=[
        >>>         'echo Hello {{ input.name }}',
        >>>         'echo Home directory is: $HOME_DIR',
        >>>     ]
        >>> )
        >>> runner.register(hello)
    """

    _pids: List[int] = []
    _global_state = CmdGlobalState()

    def __init__(
        self,
        name: str,
        group: Optional[Group] = None,
        inputs: Iterable[AnyInput] = [],
        envs: Iterable[Env] = [],
        env_files: Iterable[EnvFile] = [],
        icon: Optional[str] = None,
        color: Optional[str] = None,
        description: str = "",
        executable: Optional[str] = None,
        cmd: CmdVal = "",
        cmd_path: CmdVal = "",
        cwd: Optional[Union[str, pathlib.Path]] = None,
        upstreams: Iterable[AnyTask] = [],
        on_triggered: Optional[OnTriggered] = None,
        on_waiting: Optional[OnWaiting] = None,
        on_skipped: Optional[OnSkipped] = None,
        on_started: Optional[OnStarted] = None,
        on_ready: Optional[OnReady] = None,
        on_retry: Optional[OnRetry] = None,
        on_failed: Optional[OnFailed] = None,
        checkers: Iterable[AnyTask] = [],
        checking_interval: Union[float, int] = 0,
        retry: int = 2,
        retry_interval: Union[float, int] = 1,
        max_output_line: int = 1000,
        max_error_line: int = 1000,
        preexec_fn: Optional[Callable[[], Any]] = os.setsid,
        should_execute: Union[bool, str, Callable[..., bool]] = True,
        return_upstream_result: bool = False,
    ):
        BaseTask.__init__(
            self,
            name=name,
            group=group,
            inputs=inputs,
            envs=envs,
            env_files=env_files,
            icon=icon,
            color=color,
            description=description,
            upstreams=upstreams,
            on_triggered=on_triggered,
            on_waiting=on_waiting,
            on_skipped=on_skipped,
            on_started=on_started,
            on_ready=on_ready,
            on_retry=on_retry,
            on_failed=on_failed,
            checkers=checkers,
            checking_interval=checking_interval,
            retry=retry,
            retry_interval=retry_interval,
            should_execute=should_execute,
            return_upstream_result=return_upstream_result,
        )
        max_output_line = max_output_line if max_output_line > 0 else 1
        max_error_line = max_error_line if max_error_line > 0 else 1
        self._cmd = cmd
        self._cmd_path = cmd_path
        self.__set_cwd(cwd)
        self._max_output_size = max_output_line
        self._max_error_size = max_error_line
        self._output_buffer: Iterable[str] = []
        self._error_buffer: Iterable[str] = []
        if executable is None and default_shell != "":
            executable = default_shell
        self._executable = executable
        self._process: Optional[asyncio.subprocess.Process]
        self._preexec_fn = preexec_fn

    def copy(self) -> TCmdTask:
        return super().copy()

    def set_cwd(self, cwd: Union[str, pathlib.Path]):
        self.__set_cwd(cwd)

    def __set_cwd(self, cwd: Optional[Union[str, pathlib.Path]]):
        if cwd is None:
            self._cwd: Union[str, pathlib.Path] = os.getcwd()
            return
        self._cwd: Union[str, pathlib.Path] = os.path.abspath(cwd)

    def to_function(
        self,
        env_prefix: str = "",
        raise_error: bool = True,
        is_async: bool = False,
        show_done_info: bool = True,
    ) -> Callable[..., CmdResult]:
        return super().to_function(env_prefix, raise_error, is_async, show_done_info)

    def print_result(self, result: CmdResult):
        if result.output == "":
            return
        print(result.output)

    def inject_envs(self):
        super().inject_envs()
        input_map = self.get_input_map()
        for task_input in self._get_combined_inputs():
            input_key = to_variable_name(task_input.get_name())
            input_value = input_map.get(input_key)
            env_name = "_INPUT_" + input_key.upper()
            should_render = task_input.should_render()
            self.add_env(
                Env(
                    name=env_name,
                    os_name="",
                    default=str(input_value),
                    should_render=should_render,
                )
            )

    async def run(self, *args: Any, **kwargs: Any) -> CmdResult:
        cmd = self.get_cmd_script(*args, **kwargs)
        self.print_out_dark("Run script: " + self.__get_multiline_repr(cmd))
        self.print_out_dark("Working directory: " + self._cwd)
        self._output_buffer = []
        self._error_buffer = []
        process = await asyncio.create_subprocess_shell(
            cmd,
            cwd=self._cwd,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
            env=self.get_env_map(),
            shell=True,
            executable=self._executable,
            close_fds=True,
            preexec_fn=self._preexec_fn,
            bufsize=0,
        )
        self._set_task_pid(process.pid)
        self._pids.append(process.pid)
        self._process = process
        try:
            signal.signal(signal.SIGINT, self.__on_kill)
            signal.signal(signal.SIGTERM, self.__on_kill)
        except Exception as e:
            self.print_err(e)
        atexit.register(self.__on_exit)
        await self.__wait_process(process)
        self.log_info("Process completed")
        atexit.unregister(self.__on_exit)
        output = "\n".join(self._output_buffer)
        error = "\n".join(self._error_buffer)
        # get return code
        return_code = process.returncode
        self.log_info(f"Exit status: {return_code}")
        if return_code != 0 and not self._global_state.is_killed_by_signal:
            raise Exception(f"Process {self._name} exited ({return_code}): {error}")
        self.set_task_xcom(key="output", value=output)
        self.set_task_xcom(key="error", value=error)
        return CmdResult(output, error)

    def _should_attempt(self) -> bool:
        if self._global_state.no_more_attempt:
            return False
        return super()._should_attempt()

    def _is_last_attempt(self) -> bool:
        if self._global_state.no_more_attempt:
            return True
        return super()._is_last_attempt()

    def __on_kill(self, signum: Any, frame: Any):
        self._global_state.no_more_attempt = True
        self._global_state.is_killed_by_signal = True
        self.print_out_dark(f"Getting signal {signum}")
        for pid in self._pids:
            self.__kill_by_pid(pid)
        tasks = asyncio.all_tasks()
        for task in tasks:
            try:
                task.cancel()
            except Exception as e:
                self.print_err(e)
        time.sleep(0.3)
        self.print_out_dark(f"Exiting with signal {signum}")
        sys.exit(signum)

    def __on_exit(self):
        self._global_state.no_more_attempt = True
        self.__kill_by_pid(self._process.pid)

    def __kill_by_pid(self, pid: int):
        """
        Kill a pid, gracefully
        """
        try:
            process_ever_exists = False
            if self.__is_process_exist(pid):
                process_ever_exists = True
                self.print_out_dark(f"Send SIGTERM to process {pid}")
                os.killpg(os.getpgid(pid), signal.SIGTERM)
                time.sleep(0.3)
            if self.__is_process_exist(pid):
                self.print_out_dark(f"Send SIGINT to process {pid}")
                os.killpg(os.getpgid(pid), signal.SIGINT)
                time.sleep(0.3)
            if self.__is_process_exist(pid):
                self.print_out_dark(f"Send SIGKILL to process {pid}")
                os.killpg(os.getpgid(pid), signal.SIGKILL)
            if process_ever_exists:
                self.print_out_dark(f"Process {pid} is killed successfully")
        except Exception:
            self.log_error(f"Cannot kill process {pid}")

    def __is_process_exist(self, pid: int) -> bool:
        try:
            os.killpg(os.getpgid(pid), 0)
            return True
        except ProcessLookupError:
            return False

    async def __wait_process(self, process: asyncio.subprocess.Process):
        # Create queue
        stdout_queue = asyncio.Queue()
        stderr_queue = asyncio.Queue()
        # Read from streams and put into queue
        stdout_process = asyncio.create_task(
            self.__queue_stream(process.stdout, stdout_queue)
        )
        stderr_process = asyncio.create_task(
            self.__queue_stream(process.stderr, stderr_queue)
        )
        # Handle messages in queue
        stdout_log_process = asyncio.create_task(
            self.__log_from_queue(
                stdout_queue, self.print_out, self._output_buffer, self._max_output_size
            )
        )
        stderr_log_process = asyncio.create_task(
            self.__log_from_queue(
                stderr_queue, self.print_err, self._error_buffer, self._max_error_size
            )
        )
        # wait process
        await process.wait()
        # wait reader and logger
        await stdout_process
        await stderr_process
        await stdout_queue.put(None)
        await stderr_queue.put(None)
        await stdout_log_process
        await stderr_log_process

    def get_cmd_script(self, *args: Any, **kwargs: Any) -> str:
        return self._create_cmd_script(self._cmd_path, self._cmd, *args, **kwargs)

    def _create_cmd_script(
        self, cmd_path: CmdVal, cmd: CmdVal, *args: Any, **kwargs: Any
    ) -> str:
        if not isinstance(cmd_path, str) or cmd_path != "":
            if callable(cmd_path):
                return self.__get_rendered_cmd_path(cmd_path(*args, **kwargs))
            return self.__get_rendered_cmd_path(cmd_path)
        if callable(cmd):
            return self.__get_rendered_cmd(cmd(*args, **kwargs))
        return self.__get_rendered_cmd(cmd)

    def __get_rendered_cmd_path(self, cmd_path: Union[str, Iterable[str]]) -> str:
        if isinstance(cmd_path, str):
            return self.render_file(cmd_path)
        return "\n".join([self.render_file(cmd_path_str) for cmd_path_str in cmd_path])

    def __get_rendered_cmd(
        self, cmd: Union[JinjaTemplate, Iterable[JinjaTemplate]]
    ) -> str:
        if isinstance(cmd, str):
            return self.render_str(cmd)
        return self.render_str("\n".join(list(cmd)))

    async def __queue_stream(self, stream, queue: asyncio.Queue):
        while True:
            try:
                line = await stream.readline()
            except Exception as e:
                line = str(e)
            if not line:
                break
            await queue.put(line)

    async def __log_from_queue(
        self,
        queue: asyncio.Queue,
        print_log: Callable[[str], None],
        buffer: Iterable[str],
        max_size: int,
    ):
        while True:
            line = await queue.get()
            if not line:
                break
            line_str = line.decode("utf-8").rstrip()
            self.__add_to_buffer(buffer, max_size, line_str)
            _reset_stty()
            print_log(line_str)
            _reset_stty()

    def __add_to_buffer(self, buffer: Iterable[str], max_size: int, new_line: str):
        if len(buffer) >= max_size:
            buffer.pop(0)
        buffer.append(new_line)

    def __get_multiline_repr(self, text: str) -> str:
        lines_repr: Iterable[str] = []
        lines = text.split("\n")
        if len(lines) == 1:
            return lines[0]
        for index, line in enumerate(lines):
            line_number_repr = str(index + 1).rjust(4, "0")
            lines_repr.append(f"        {line_number_repr} | {line}")
        return "\n" + "\n".join(lines_repr)



from abc import ABC, abstractmethod

from component.repo import DBEntityMixin, DBRepo, Repo
from module.auth.integration import Base
from module.auth.schema.permission import Permission, PermissionData
from sqlalchemy import Column, String


class DBEntityPermission(Base, DBEntityMixin):
    class Config:
        orm_mode = True
        from_attributes = True

    __tablename__ = "permissions"
    name = Column(String)
    description = Column(String)


class PermissionRepo(Repo[Permission, PermissionData], ABC):
    @abstractmethod
    async def get_by_name(self, name: str) -> Permission:
        pass


class PermissionDBRepo(DBRepo[Permission, PermissionData], PermissionRepo):
    schema_cls = Permission
    db_entity_cls = DBEntityPermission

    async def get_by_name(self, name: str) -> Permission:
        """
        Find permission by name.
        """
        db = self._get_db_session()
        try:
            search_filter = DBEntityPermission.name == name
            db_permission = self._get_one_by_criterion(db, search_filter)
            return self._db_entity_to_schema(db, db_permission)
        finally:
            db.close()



from typing import Any, List

from zrb import Task

###############################################################################
# Functions
###############################################################################


def should_start_local_microservices(*args: Any, **kwargs: Any) -> bool:
    if not kwargs.get("local_snake_zrb_app_name", True):
        return False
    run_mode = kwargs.get("snake_zrb_app_name_run_mode", "monolith")
    return run_mode == "microservices"


def get_support_container_compose_profiles(*args: Any, **kwargs: Any) -> List[str]:
    task: Task = kwargs.get("_task")
    env_map = task.get_env_map()
    compose_profiles: List[str] = []
    broker_type = env_map.get("APP_BROKER_TYPE", "rabbitmq")
    if broker_type in ["rabbitmq", "kafka"]:
        compose_profiles.append(broker_type)
    if kwargs.get("enable_snake_zrb_app_name_monitoring", False):
        compose_profiles.append("monitoring")
    return compose_profiles


def get_container_compose_profiles(*args: Any, **kwargs: Any) -> str:
    compose_profiles = get_support_container_compose_profiles(*args, **kwargs)
    compose_profiles.append(
        kwargs.get("snake_zrb_app_name_run_mode", "monolith"),
    )
    return compose_profiles


def activate_support_compose_profile(*args: Any, **kwargs: Any) -> str:
    compose_profiles = get_support_container_compose_profiles(*args, **kwargs)
    compose_profile_str = ",".join(compose_profiles)
    return f"export COMPOSE_PROFILES={compose_profile_str}"


def should_start_support_container(*args: Any, **kwargs: Any) -> bool:
    if not kwargs.get("local_snake_zrb_app_name", True):
        return False
    compose_profiles = get_support_container_compose_profiles(*args, **kwargs)
    return len(compose_profiles) > 0


def should_start_local_monolith(*args: Any, **kwargs: Any) -> bool:
    if not kwargs.get("local_snake_zrb_app_name", True):
        return False
    return kwargs.get("snake_zrb_app_name_run_mode", "monolith") == "monolith"


def activate_selected_compose_profile(*args: Any, **kwargs: Any) -> str:
    compose_profiles = get_container_compose_profiles(*args, **kwargs)
    compose_profile_str = ",".join(compose_profiles)
    return f"export COMPOSE_PROFILES={compose_profile_str}"


def activate_all_compose_profile(*args: Any, **kwargs: Any) -> str:
    compose_profile_str = ",".join(
        ["monitoring", "monolith", "microservices", "kafka", "rabbitmq"]
    )
    return f"export COMPOSE_PROFILES={compose_profile_str}"


def should_start_container(*args: Any, **kwargs: Any) -> bool:
    return kwargs.get("local_snake_zrb_app_name", True)



from logging import Logger

from component.error import HTTPAPIException
from component.messagebus import Publisher
from component.rpc import Caller
from fastapi import Depends, FastAPI
from module.auth.component import Authorizer
from module.auth.integration import access_token_scheme
from module.auth.schema.token import AccessTokenData
from module.log.schema.activity import Activity, ActivityResult


def register_api(
    logger: Logger,
    app: FastAPI,
    authorizer: Authorizer,
    rpc_caller: Caller,
    publisher: Publisher,
):
    logger.info('🥪 Register API for "log.activity"')

    @app.get("/api/v1/log/activities", response_model=ActivityResult)
    async def get_activitys(
        keyword: str = "",
        limit: int = 100,
        offset: int = 0,
        user_token_data: AccessTokenData = Depends(access_token_scheme),
    ):
        if not await authorizer.is_having_permission(
            user_token_data.user_id, "log:activity:get"
        ):
            raise HTTPAPIException(403, "Unauthorized")
        try:
            result_dict = await rpc_caller.call(
                "log_get_activity",
                keyword=keyword,
                criterion={},
                limit=limit,
                offset=offset,
                user_token_data=user_token_data.model_dump(),
            )
            return ActivityResult(**result_dict)
        except Exception as e:
            raise HTTPAPIException(error=e)

    @app.get("/api/v1/log/activities/{id}", response_model=Activity)
    async def get_activity_by_id(
        id: str, user_token_data: AccessTokenData = Depends(access_token_scheme)
    ):
        if not await authorizer.is_having_permission(
            user_token_data.user_id, "log:activity:get_by_id"
        ):
            raise HTTPAPIException(403, "Unauthorized")
        try:
            result_dict = await rpc_caller.call(
                "log_get_activity_by_id",
                id=id,
                user_token_data=user_token_data.model_dump(),
            )
            return Activity(**result_dict)
        except Exception as e:
            raise HTTPAPIException(error=e)



from zrb import DockerComposeTask, Env, StrInput, runner
from zrb.builtin.group import project_group

from ._common import RESOURCE_DIR, local_input

###############################################################################
# 🔤 Input Definitions
###############################################################################

image_input = StrInput(
    name="kebab-zrb-app-name-image",
    description='Image name of "kebab-zrb-app-name"',
    prompt='Image name of "kebab-zrb-app-name"',
    default="zrb-app-image-name:latest",
)


###############################################################################
# 🌱 Env Definitions
###############################################################################

image_env = Env(
    name="IMAGE",
    os_name="CONTAINER_ZRB_ENV_PREFIX_IMAGE",
    default="{{input.snake_zrb_app_name_image}}",
)

###############################################################################
# ⚙️ build-zrb-task-name-image
###############################################################################

build_snake_zrb_app_name_image = DockerComposeTask(
    icon="🏭",
    name="build-kebab-zrb-app-name-image",
    description="Build human readable zrb app name image",
    group=project_group,
    inputs=[
        local_input,
        image_input,
    ],
    envs=[image_env],
    should_execute="{{input.local_snake_zrb_app_name}}",
    cwd=RESOURCE_DIR,
    compose_cmd="build",
    compose_flags=["--no-cache"],
    compose_env_prefix="CONTAINER_ZRB_ENV_PREFIX",
)
runner.register(build_snake_zrb_app_name_image)

###############################################################################
# ⚙️ push-zrb-task-name-image
###############################################################################

push_snake_zrb_app_name_image = DockerComposeTask(
    icon="📰",
    name="push-kebab-zrb-app-name-image",
    description="Push human readable zrb app name image",
    group=project_group,
    inputs=[
        local_input,
        image_input,
    ],
    envs=[image_env],
    upstreams=[build_snake_zrb_app_name_image],
    cwd=RESOURCE_DIR,
    compose_cmd="push",
    compose_env_prefix="CONTAINER_ZRB_ENV_PREFIX",
)
runner.register(push_snake_zrb_app_name_image)



from zrb.builtin.group import docker_group
from zrb.runner import runner
from zrb.task.cmd_task import CmdTask

###############################################################################
# Task Definitions
###############################################################################

prune = CmdTask(
    name="prune",
    group=docker_group,
    description="Prune unused images and volumes",
    cmd=[
        "docker system prune -af",
        "docker image prune -af",
        "docker system prune -af --volumes",
        "docker system df",
    ],
)
runner.register(prune)



import asyncio
import inspect
from typing import Awaitable, Callable


def create_task(awaitable: Awaitable, on_error: Callable) -> asyncio.Task:
    async def critical_task(awaitable):
        try:
            return await awaitable
        except (asyncio.CancelledError, GeneratorExit, Exception) as e:
            if inspect.iscoroutinefunction(on_error):
                return await on_error(e)
            return on_error(e)

    return asyncio.create_task(critical_task(awaitable))



import copy
import os
import re
from typing import List, Mapping

import jsons
from dotenv import dotenv_values

NON_ALPHA_NUM = re.compile(r"[^a-zA-Z0-9]+")
CURRENT_DIR: str = os.path.dirname(__file__)
MODE: str = os.getenv("MODE", "monolith")
NAMESPACE: str = os.getenv("NAMESPACE", "default")

ENABLE_MONITORING: bool = os.getenv("ENABLE_MONITORING", "0") in [
    "1",
    "true",
    "True",
    "Yes",
    "yes",
    "Y",
    "y",
]

MODULE_JSON_STR: str = os.getenv("MODULES", "[]")
MODULES: List[str] = jsons.loads(MODULE_JSON_STR)

APP_DIR: str = os.path.abspath(os.path.join(CURRENT_DIR, "..", "src"))
TEMPLATE_ENV_FILE_NAME: str = os.path.join(APP_DIR, "template.env")
TEMPLATE_ENV_MAP: Mapping[str, str] = {
    key: os.getenv(key, default_value)
    for key, default_value in dotenv_values(TEMPLATE_ENV_FILE_NAME).items()
}

BROKER_TYPE: str = os.getenv("BROKER_TYPE", "")
if BROKER_TYPE == "":
    BROKER_TYPE = TEMPLATE_ENV_MAP.get("APP_BROKER_TYPE", "rabbitmq")

RABBITMQ_AUTH_USERNAME: str = os.getenv("RABBITMQ_AUTH_USERNAME", "root")
RABBITMQ_AUTH_PASSWORD: str = os.getenv("RABBITMQ_AUTH_PASSWORD", "toor")

REDPANDA_AUTH_SASL_ENABLED: bool = (
    os.getenv("REDPANDA_AUTH_SASL_ENABLED", "true").lower() == "true"
)
REDPANDA_AUTH_MECHANISM: str = os.getenv("REDPANDA_AUTH_MECHANISM", "SCRAM-SHA-512")
REDPANDA_AUTH_USER_NAME: str = os.getenv("REDPANDA_AUTH_USER_NAME", "root")
REDPANDA_AUTH_USER_PASSWORD: str = os.getenv("REDPANDA_AUTH_USER_PASSWORD", "toor")

POSTGRESQL_AUTH_POSTGRES_PASSWORD: str = os.getenv("POSTGRESQL_AUTH_POSTGRES_PASSWORD")
POSTGRESQL_AUTH_USERNAME: str = os.getenv("POSTGRESQL_AUTH_USERNAME", "root")
POSTGRESQL_AUTH_PASSWORD: str = os.getenv("POSTGRESQL_AUTH_PASSWORD", "toor")
POSTGRESQL_DB: str = os.getenv("POSTGRESQL_DB", "snake_zrb_app_name")

SIGNOZ_CLICKHOUSE_NAMESPACE: str = os.getenv("SIGNOZ_CLICKHOUSE_NAMESPACE", NAMESPACE)
SIGNOZ_CLICKHOUSE_USER: str = os.getenv("SIGNOZ_CLICKHOUSE_USER", "root")
SIGNOZ_CLICKHOUSE_PASSWORD: str = os.getenv("SIGNOZ_CLICKHOUSE_PASSWORD", "toor")


def get_app_monolith_env_map(
    template_env_map: Mapping[str, str], modules: List[str]
) -> Mapping[str, str]:
    env_map = copy.deepcopy(template_env_map)
    env_map[
        "APP_RMQ_CONNECTION"
    ] = f"amqp://{RABBITMQ_AUTH_USERNAME}:{RABBITMQ_AUTH_PASSWORD}@rabbitmq"  # noqa
    env_map["APP_KAFKA_BOOTSTRAP_SERVERS"] = "redpanda:9092"
    env_map["APP_KAFKA_SASL_MECHANISM"] = REDPANDA_AUTH_MECHANISM
    env_map["APP_KAFKA_SASL_USER"] = REDPANDA_AUTH_USER_NAME
    env_map["APP_KAFKA_SASL_PASSWORD"] = REDPANDA_AUTH_USER_PASSWORD
    env_map[
        "APP_DB_CONNECTION"
    ] = f"postgresql+psycopg2://{POSTGRESQL_AUTH_USERNAME}:{POSTGRESQL_AUTH_PASSWORD}@postgresql:5432/{POSTGRESQL_DB}"  # noqa
    for module_name in modules:
        env_name = get_module_flag_env_name(module_name)
        env_map[env_name] = "true"
    env_map["APP_ENABLE_EVENT_HANDLER"] = "true"
    env_map["APP_ENABLE_RPC_SERVER"] = "true"
    env_map["APP_ENABLE_FRONTEND"] = "true"
    env_map["APP_ENABLE_API"] = "true"
    env_map["APP_ENABLE_OTEL"] = "true" if ENABLE_MONITORING else "false"
    env_map[
        "APP_OTEL_EXPORTER_OTLP_ENDPOINT"
    ] = "http://signoz-otel-collector:4317"  # noqa
    return env_map


def get_app_gateway_env_map(
    template_env_map: Mapping[str, str], modules: List[str]
) -> Mapping[str, str]:
    env_map = get_app_monolith_env_map(template_env_map, modules)
    for module_name in modules:
        env_name = get_module_flag_env_name(module_name)
        env_map[env_name] = "true"
    env_map["APP_ENABLE_EVENT_HANDLER"] = "false"
    env_map["APP_ENABLE_RPC_SERVER"] = "false"
    env_map["APP_ENABLE_FRONTEND"] = "true"
    env_map["APP_ENABLE_API"] = "true"
    env_map["APP_DB_AUTO_MIGRATE"] = "false"
    return env_map


def get_app_service_env_map(
    template_env_map: Mapping[str, str], modules: List[str], current_module: str
) -> Mapping[str, str]:
    env_map = get_app_monolith_env_map(template_env_map, modules)
    for module_name in modules:
        env_name = get_module_flag_env_name(module_name)
        if module_name == current_module:
            env_map[env_name] = "true"
            continue
        env_map[env_name] = "false"
    env_map["APP_ENABLE_EVENT_HANDLER"] = "true"
    env_map["APP_ENABLE_RPC_SERVER"] = "true"
    env_map["APP_ENABLE_FRONTEND"] = "false"
    env_map["APP_ENABLE_API"] = "false"
    return env_map


def get_module_flag_env_name(module_name: str) -> str:
    upper_snake_module_name = to_snake_case(module_name).upper()
    return f"APP_ENABLE_{upper_snake_module_name}_MODULE"


def to_kebab_case(text: str) -> str:
    text = to_alphanum(text)
    return "-".join([x.lower() for x in text.split(" ")])


def to_snake_case(text: str) -> str:
    text = to_alphanum(text)
    return "_".join([x.lower() for x in text.split(" ")])


def to_alphanum(text: str) -> str:
    return NON_ALPHA_NUM.sub(" ", text)



import hashlib

import aiofiles

from zrb.builtin.group import md5_group
from zrb.helper.typing import Any
from zrb.runner import runner
from zrb.task.decorator import python_task
from zrb.task_input.str_input import StrInput

###############################################################################
# Task Definitions
###############################################################################


@python_task(
    name="hash",
    group=md5_group,
    inputs=[StrInput(name="text", shortcut="t", description="Text", default="")],
    description="Hash md5",
    retry=0,
    runner=runner,
)
async def hash_text(*args: str, **kwargs: Any):
    text: str = kwargs.get("text", "")
    hashed_text = hashlib.md5(text.encode()).hexdigest()
    return hashed_text


@python_task(
    name="sum",
    group=md5_group,
    inputs=[StrInput(name="file", shortcut="f", description="File", default="")],
    description="Sum md5 file",
    retry=0,
    runner=runner,
)
async def sum_file(*args: str, **kwargs: Any):
    file_path: str = kwargs.get("file", "")
    async with aiofiles.open(file_path, mode="rb") as file:
        contents = await file.read()
    hashed_text = hashlib.md5(contents).hexdigest()
    return hashed_text



from zrb.builtin.group import process_group
from zrb.runner import runner
from zrb.task.cmd_task import CmdTask
from zrb.task_input.int_input import IntInput
from zrb.task_input.str_input import StrInput

###############################################################################
# Task Definitions
###############################################################################

get_pid_by_port = CmdTask(
    name="get-pid-by-port",
    group=process_group,
    description="Get PID by port",
    inputs=[
        IntInput(
            name="port",
            shortcut="p",
            default="8080",
            description="Port to be checked",
            prompt="Port to be checked",
        )
    ],
    cmd="lsof -i :{{ input.port }} -n -P | awk 'NR>1 {print $2}'",
    checking_interval=3,
)
runner.register(get_pid_by_port)

get_pid_by_name = CmdTask(
    name="get-pid-by-name",
    group=process_group,
    description="Get PID by name",
    inputs=[
        StrInput(
            name="name",
            shortcut="n",
            default="python",
            description="Process name to be checked",
            prompt="Process name to be checked",
        )
    ],
    cmd="pgrep {{ input.name }}",
    checking_interval=3,
)
runner.register(get_pid_by_name)



import asyncio
import inspect
import logging
from typing import Any, Callable, Mapping, Optional

import aiormq
from component.messagebus.messagebus import (
    Consumer,
    MessageSerializer,
    TEventHandler,
    must_get_message_serializer,
)
from component.messagebus.rabbitmq.admin import RMQAdmin, must_get_rmq_admin


class RMQConsumer(Consumer):
    def __init__(
        self,
        logger: logging.Logger,
        connection_string: str,
        serializer: Optional[MessageSerializer] = None,
        rmq_admin: Optional[RMQAdmin] = None,
        retry: int = 5,
        retry_interval: int = 10,
        prefetch_count: int = 20,
        identifier="rmq-consumer",
    ):
        self.logger = logger
        self.rmq_admin = must_get_rmq_admin(
            logger=logger, rmq_admin=rmq_admin, connection_string=connection_string
        )
        self.connection_string = connection_string
        self.connection: Optional[aiormq.Connection] = None
        self.channel: Optional[aiormq.Channel] = None
        self.serializer = must_get_message_serializer(serializer)
        self.retry = retry
        self.retry_interval = retry_interval
        self.prefetch_count = prefetch_count
        self._handlers: Mapping[str, TEventHandler] = {}
        self._is_start_triggered = False
        self._is_stop_triggered = False
        self.identifier = identifier

    def register(self, event_name: str) -> Callable[[TEventHandler], Any]:
        def wrapper(handler: TEventHandler):
            self.logger.warning(
                f'🐰 [{self.identifier}] Register handler for "{event_name}"'
            )
            self._handlers[event_name] = handler
            return handler

        return wrapper

    async def start(self):
        if self._is_start_triggered:
            return
        self._is_start_triggered = True
        return await self._start(self.retry)

    async def stop(self):
        if self._is_stop_triggered:
            return
        self._is_stop_triggered = True
        await self._disconnect()

    async def _start(self, retry: int):
        try:
            await self._connect()
            event_names = list(self._handlers.keys())
            await self.rmq_admin.create_events(event_names)
            f'🐰 [{self.identifier}] Listening from "{event_names}"'
            for event_name in event_names:
                queue_name = self.rmq_admin.get_queue_name(event_name)
                on_message = self._create_consumer_callback(self.channel, event_name)
                await self.channel.basic_consume(
                    queue=queue_name, consumer_callback=on_message
                )
            retry = self.retry
            while not self._is_stop_triggered:
                await asyncio.sleep(0.01)
                if not self._is_stop_triggered and (
                    self.connection is None or self.connection.is_closed
                ):
                    raise Exception("Rabbitmq connection is closed")
        except (asyncio.CancelledError, GeneratorExit, Exception) as e:
            if retry > 0:
                self.logger.error(f"🐰 [{self.identifier}]", exc_info=True)
            if retry == 0:
                self.logger.error(
                    f"🐰 [{self.identifier}] Failed to consume message after "
                    + f"{self.retry} attempts"
                )
                raise e
            await self._disconnect()
            await asyncio.sleep(self.retry_interval)
            await self._start(retry - 1)
        finally:
            await self._disconnect()

    async def _connect(self):
        try:
            connection_created = False
            if self.connection is None or self.connection.is_closed:
                self.logger.info(f"🐰 [{self.identifier}] Create consumer connection")
                self.connection = await aiormq.connect(self.connection_string)
                self.logger.info(f"🐰 [{self.identifier}] Consumer connection created")
                connection_created = True
            if connection_created or self.channel is None or self.channel.is_closed:
                self.logger.info(f"🐰 [{self.identifier}] Get consumer channel")
                self.channel = await self.connection.channel()
                await self.channel.basic_qos(prefetch_count=self.prefetch_count)
                self.logger.info(f"🐰 [{self.identifier}] Consumer channel created")
        except (asyncio.CancelledError, GeneratorExit, Exception):
            self.logger.error(f"🐰 [{self.identifier}]", exc_info=True)
            raise Exception("Cannot connect")

    async def _disconnect(self):
        try:
            if self.channel is not None and not self.channel.is_closed:
                self.logger.info(f"🐰 [{self.identifier}] Close consumer channel")
                await self.channel.close()
                self.logger.info(f"🐰 [{self.identifier}] Consumer channel closed")
        except (asyncio.CancelledError, GeneratorExit, Exception):
            self.logger.error(f"🐰 [{self.identifier}]", exc_info=True)
        try:
            if self.connection is not None and not self.connection.is_closed:
                self.logger.info(f"🐰 [{self.identifier}] Close consumer connection")
                await self.connection.close()
                self.logger.info(f"🐰 [{self.identifier}] Consumer connection closed")
        except (asyncio.CancelledError, GeneratorExit, Exception):
            self.logger.error(f"🐰 [{self.identifier}]", exc_info=True)
        self.connection = None
        self.channel = None

    def _create_consumer_callback(
        self,
        channel: aiormq.Channel,
        event_name: str,
    ) -> Callable[[Any], Any]:
        async def on_message(message):
            try:
                decoded_value = self.serializer.decode(event_name, message.body)
                handler = self._handlers.get(event_name)
                queue_name = self.rmq_admin.get_queue_name(event_name)
                self.logger.info(
                    f'🐰 [{self.identifier}] Consume from "{queue_name}": '
                    + f"{decoded_value}"
                )
                await self._run_handler(handler, decoded_value)
                await channel.basic_ack(message.delivery_tag)
            except (asyncio.CancelledError, GeneratorExit, Exception):
                self.logger.error(f"🐰 [{self.identifier}]", exc_info=True)

        return on_message

    async def _run_handler(self, message_handler: TEventHandler, decoded_value: Any):
        if inspect.iscoroutinefunction(message_handler):
            return asyncio.create_task(message_handler(decoded_value))
        return message_handler(decoded_value)



import libcst as cst

from zrb.helper.typecheck import typechecked


@typechecked
class AppendCodeTransformer(cst.CSTTransformer):
    def __init__(self, function_name: str, new_code: str):
        self.function_name = function_name
        self.new_code = new_code
        super().__init__()

    def leave_FunctionDef(
        self, original_node: cst.FunctionDef, updated_node: cst.FunctionDef
    ) -> cst.FunctionDef:
        if updated_node.name.value == self.function_name:
            new_body = updated_node.body.body + (cst.parse_statement(self.new_code),)
            return updated_node.with_changes(body=cst.IndentedBlock(body=new_body))
        return updated_node


@typechecked
def append_code_to_function(code: str, function_name: str, new_code: str) -> str:
    module = cst.parse_module(code)
    transformed_module = module.visit(AppendCodeTransformer(function_name, new_code))
    return transformed_module.code



from zrb.helper.typecheck import typechecked
from zrb.task.task import Task


@typechecked
def show_lines(task: Task, *lines: str):
    separator = "\n    "
    task.print_out("\n" + separator + separator.join(lines) + "\n", trim_message=False)



from integration.messagebus import publisher
from module.auth.entity.group.model import GroupModel
from module.auth.integration.repo.group_repo import group_repo

group_model: GroupModel = GroupModel(group_repo, publisher)



from zrb import CmdTask, runner
from zrb.builtin.group import project_group

from .container import start_snake_zrb_app_name_container

###############################################################################
# ⚙️ start-kebab-zrb-task-name
###############################################################################

start_snake_zrb_app_name = CmdTask(
    icon="🚤",
    name="start-kebab-zrb-app-name",
    description="Start human readable zrb app name (as container)",
    group=project_group,
    upstreams=[start_snake_zrb_app_name_container],
)
runner.register(start_snake_zrb_app_name)



from zrb.builtin import (
    base64,
    devtool,
    docker,
    env,
    eval,
    explain,
    generator,
    git,
    md5,
    process,
    project,
    say,
    schedule,
    ubuntu,
    update,
    version,
    watch_changes,
)

assert base64
assert devtool
assert docker
assert env
assert eval
assert explain
assert generator
assert git
assert md5
assert process
assert project
assert ubuntu
assert update
assert say
assert schedule
assert version
assert watch_changes



import inspect
import re


def get_markdown_from_docstring(cls) -> str:
    """
    Convert Google Style docstrings of a class and its methods to Markdown.

    Args:
        cls (class): The class whose docstrings are to be converted.

    Returns:
        str: The converted Markdown text.
    """
    markdown = f"## `{cls.__name__}`\n\n"
    cls_doc = get_doc(cls)
    markdown += parse_docstring(cls_doc) + "\n"
    for method_name, _ in inspect.getmembers(cls, predicate=inspect.isfunction):  # noqa
        if method_name.startswith("__"):
            # Don't parse private or protected function
            continue
        markdown += f"\n### `{cls.__name__}.{method_name}`\n\n"
        method_doc = get_method_doc(cls, method_name)
        markdown += parse_docstring(method_doc) + "\n"
    return markdown


def get_method_doc(cls, method_name: str) -> str:
    if not hasattr(cls, method_name):
        return None
    method = getattr(cls, method_name)
    if not method.__doc__ and hasattr(cls, "__bases__"):
        for parent in cls.__bases__:
            parent_method_doc = get_method_doc(parent, method_name)
            if parent_method_doc:
                return parent_method_doc
    return method.__doc__


def get_doc(cls) -> str:
    if not cls.__doc__ and hasattr(cls, "__bases__"):
        for parent in cls.__bases__:
            parent_doc = get_doc(parent)
            if parent_doc:
                return parent_doc
    return cls.__doc__


def parse_docstring(docstring: str) -> str:
    if not docstring:
        return "No documentation available.\n"
    # Split the docstring into lines
    lines = docstring.strip().split("\n")
    line_length = len(lines)
    # Process each line
    markdown_lines = []
    section = ""
    is_previous_code = False
    has_unclosed_backtick = False
    for line_index, line in enumerate(lines):
        line = line.strip()
        is_code = line.startswith(">>> ")
        is_last_line = line_index == line_length - 1
        is_new_section = False
        # Add code backticks
        if is_code and not is_previous_code:
            markdown_lines.append("```python")
            has_unclosed_backtick = True
        elif section == "examples" and is_previous_code and not is_code:
            markdown_lines.append("```")
            markdown_lines.append("")
            markdown_lines.append("```")
            has_unclosed_backtick = True
        elif is_previous_code and not is_code:
            markdown_lines.append("````")
            markdown_lines.append("")
            has_unclosed_backtick = False
        # Handle lines
        if is_code:
            markdown_lines.append(line[4:])
        elif (
            line.startswith("Attributes:")
            or line.startswith("Attribute:")
            or line.startswith("Attrs:")
            or line.startswith("Attr:")
        ):
            markdown_lines.append("__Attributes:__\n")
            section = "attributes"
            is_new_section = True
        elif (
            line.startswith("Args:")
            or line.startswith("Arg:")
            or line.startswith("Arguments:")
            or line.startswith("Argument:")
        ):
            markdown_lines.append("__Arguments:__\n")
            section = "args"
            is_new_section = True
        elif line.startswith("Returns:"):
            markdown_lines.append("__Returns:__\n")
            section = "returns"
            is_new_section = True
        elif line.startswith("Examples:") or line.startswith("Examples:"):
            markdown_lines.append("__Examples:__\n")
            section = "examples"
            is_new_section = True
        elif line == "```":
            markdown_lines.append(line)
        elif section == "args" or section == "attributes":
            named_param_match = re.match(r"^(\w+)\s+\((.+)\):(.+)$", line)
            if named_param_match:
                param_name, param_type, param_desc = named_param_match.groups()
                markdown_lines.append(
                    f"- `{param_name}` (`{param_type}`): {param_desc.strip()}"
                )
            else:
                markdown_lines.append(line)
        elif section == "returns":
            return_match = re.match(r"^(.+):(.+)$", line)
            if return_match:
                param_type, param_desc = return_match.groups()
                markdown_lines.append(f"`{param_type}`: {param_desc.strip()}")
            else:
                markdown_lines.append(line)
        else:
            markdown_lines.append(line)
        is_previous_code = is_code
        if (is_new_section or is_last_line) and has_unclosed_backtick:
            markdown_lines.append("```")
            markdown_lines.append("")
            has_unclosed_backtick = False
    return "\n".join(markdown_lines)



import time

from config import app_db_connection, app_db_engine_show_log
from integration.log import logger
from sqlalchemy import Engine, create_engine, event


@event.listens_for(Engine, "before_cursor_execute")
def before_cursor_execute(conn, cursor, statement, parameters, context, executemany):
    context._query_start_time = time.time()
    one_line_statement = " ".join(statement.split("\n"))
    logger.info(f"💽 [start query] SQL:`{one_line_statement}`, parameters {parameters}")


@event.listens_for(Engine, "after_cursor_execute")
def after_cursor_execute(conn, cursor, statement, parameters, context, executemany):
    total = time.time() - context._query_start_time
    one_line_statement = " ".join(statement.split("\n"))
    logger.info(
        " ".join(
            [
                f"💽 [query completed] Elapsed time: {total} ms,",
                f"SQL: `{one_line_statement}`,",
                f"parameters {parameters}",
            ]
        )
    )


engine: Engine = create_engine(app_db_connection, echo=app_db_engine_show_log)



import libcst as cst

from zrb.helper.typecheck import typechecked


@typechecked
class AddArgumentTransformer(cst.CSTTransformer):
    def __init__(self, function_name: str, argument_name: str, argument_type: str):
        self.function_name = function_name
        self.argument_name = argument_name
        self.argument_type = argument_type
        super().__init__()

    def leave_FunctionDef(
        self, original_node: cst.FunctionDef, updated_node: cst.FunctionDef
    ) -> cst.FunctionDef:
        if updated_node.name.value == self.function_name:
            new_params = updated_node.params.params + (
                cst.Param(
                    cst.Name(self.argument_name),
                    cst.Annotation(cst.Name(self.argument_type)),
                ),
            )
            return updated_node.with_changes(
                params=cst.Parameters(new_params),
            )
        return updated_node


@typechecked
def add_argument_to_function(
    code: str, function_name: str, argument_name: str, argument_type: str
) -> str:
    module = cst.parse_module(code)
    transformed_module = module.visit(
        AddArgumentTransformer(function_name, argument_name, argument_type)
    )
    return transformed_module.code



from zrb.helper.typing import Any
from zrb.runner import runner
from zrb.task.decorator import python_task
from zrb.task_input.str_input import StrInput

###############################################################################
# Task Definitions
###############################################################################


@python_task(
    name="eval",
    inputs=[
        StrInput(
            name="expression",
            shortcut="e",
            default="",
            description="Python expression",
        )
    ],
    description="Evaluate Python expression",
    retry=0,
    runner=runner,
)
async def evaluate(*args: str, **kwargs: Any):
    expression: str = kwargs.get("expression", "")
    return eval(expression)



import bcrypt
from module.auth.component.password_hasher.password_hasher import PasswordHasher


class BcryptPasswordHasher(PasswordHasher):
    def hash_password(self, password: str) -> str:
        return bcrypt.hashpw(password.encode("utf-8"), bcrypt.gensalt()).decode("utf-8")

    def check_password(self, password: str, hashed_password: str) -> bool:
        return bcrypt.checkpw(password.encode("utf-8"), hashed_password.encode("utf-8"))



from typing import Callable, Optional

from fastapi import Depends
from fastapi.security import OAuth2PasswordBearer
from module.auth.component.access_token.util import AccessTokenUtil
from module.auth.schema.token import AccessTokenData
from module.auth.schema.user import User
from starlette.requests import Request

AccessTokenScheme = Callable[[Request], AccessTokenData]


def create_oauth2_bearer_access_token_scheme(
    guest_user: User,
    access_token_util: AccessTokenUtil,
    token_url: str,
    token_cookie_key: str,
) -> AccessTokenScheme:
    oauth2_scheme = OAuth2PasswordBearer(tokenUrl=token_url, auto_error=False)

    async def oauth2_bearer_token_scheme(
        request: Request, token: Optional[str] = Depends(oauth2_scheme)
    ) -> AccessTokenData:
        token: Optional[str] = await oauth2_scheme(request)
        if token is None:
            token = request.cookies.get(token_cookie_key, None)
        if token is None:
            return AccessTokenData(
                user_id=guest_user.id, username=guest_user.username, expire_seconds=300
            )
        return access_token_util.decode(token, parse_expired_token=False)

    return oauth2_bearer_token_scheme



import os

from zrb import CmdTask, Env, EnvFile, IntInput, StrInput, runner
from zrb.builtin.group import project_group

from ._common import (
    APP_TEMPLATE_ENV_FILE_NAME,
    CURRENT_DIR,
    DEPLOYMENT_DIR,
    DEPLOYMENT_TEMPLATE_ENV_FILE_NAME,
)
from .image import image_env, image_input, push_snake_zrb_app_name_image

###############################################################################
# 🔤 Input Definitions
###############################################################################

replica_input = IntInput(
    name="kebab-zrb-app-name-replica",
    description='Replica of "kebab-zrb-app-name"',
    prompt='Replica of "kebab-zrb-app-name"',
    default=1,
)

pulumi_stack_input = StrInput(
    name="kebab-zrb-app-name-pulumi-stack",
    description='Pulumi stack name for "kebab-zrb-app-name"',
    prompt='Pulumi stack name for "kebab-zrb-app-name"',
    default=os.getenv("ZRB_ENV", "dev"),
)

###############################################################################
# 🌳 Env File Definitions
###############################################################################

deployment_app_env_file = EnvFile(
    path=APP_TEMPLATE_ENV_FILE_NAME, prefix="DEPLOYMENT_APP_ZRB_ENV_PREFIX"
)

deployment_config_env_file = EnvFile(
    path=DEPLOYMENT_TEMPLATE_ENV_FILE_NAME, prefix="DEPLOYMENT_CONFIG_ZRB_ENV_PREFIX"
)

###############################################################################
# 🌱 Env Definitions
###############################################################################

pulumi_backend_url_env = Env(
    name="PULUMI_BACKEND_URL",
    os_name="PULUMI_ZRB_ENV_PREFIX_BACKEND_URL",
    default=f"file://{DEPLOYMENT_DIR}/state",
)

pulumi_config_passphrase_env = Env(
    name="PULUMI_CONFIG_PASSPHRASE",
    os_name="PULUMI_ZRB_ENV_PREFIX_CONFIG_PASSPHRASE",
    default="secret",
)

deployment_replica_env = Env(
    name="REPLICA",
    os_name="DEPLOYMENT_ZRB_ENV_PREFIX",
    default="{{input.snake_zrb_app_name_replica}}",
)

###############################################################################
# ⚙️ deploy-zrb-task-name
###############################################################################

deploy_snake_zrb_app_name = CmdTask(
    icon="🚧",
    name="deploy-kebab-zrb-app-name",
    description="Deploy human readable zrb app name",
    group=project_group,
    inputs=[
        image_input,
        replica_input,
        pulumi_stack_input,
    ],
    upstreams=[push_snake_zrb_app_name_image],
    cwd=DEPLOYMENT_DIR,
    env_files=[
        deployment_config_env_file,
        deployment_app_env_file,
    ],
    envs=[
        pulumi_backend_url_env,
        pulumi_config_passphrase_env,
        image_env,
        deployment_replica_env,
    ],
    cmd_path=[
        os.path.join(CURRENT_DIR, "cmd", "pulumi-init-stack.sh"),
        os.path.join(CURRENT_DIR, "cmd", "pulumi-up.sh"),
    ],
)
runner.register(deploy_snake_zrb_app_name)

###############################################################################
# ⚙️ destroy-zrb-task-name
###############################################################################

destroy_snake_zrb_app_name = CmdTask(
    icon="💨",
    name="destroy-kebab-zrb-app-name",
    description="Remove human readable zrb app name deployment",
    group=project_group,
    inputs=[
        pulumi_stack_input,
    ],
    cwd=DEPLOYMENT_DIR,
    env_files=[
        deployment_config_env_file,
        deployment_app_env_file,
    ],
    envs=[
        pulumi_backend_url_env,
        pulumi_config_passphrase_env,
        image_env,
        deployment_replica_env,
    ],
    cmd_path=[
        os.path.join(CURRENT_DIR, "cmd", "pulumi-init-stack.sh"),
        os.path.join(CURRENT_DIR, "cmd", "pulumi-destroy.sh"),
    ],
)
runner.register(destroy_snake_zrb_app_name)



from _common import (
    NAMESPACE,
    REDPANDA_AUTH_MECHANISM,
    REDPANDA_AUTH_SASL_ENABLED,
    REDPANDA_AUTH_USER_NAME,
    REDPANDA_AUTH_USER_PASSWORD,
)
from pulumi_kubernetes.helm.v3 import Chart, LocalChartOpts


def create_redpanda() -> Chart:
    redpanda = Chart(
        "redpanda",
        LocalChartOpts(
            path="./helm-charts/redpanda",
            namespace=NAMESPACE,
            values={
                "auth": {
                    "sasl": {
                        "enabled": REDPANDA_AUTH_SASL_ENABLED,
                        "mechanism": REDPANDA_AUTH_MECHANISM,
                        "users": [
                            {
                                "name": REDPANDA_AUTH_USER_NAME,
                                "password": REDPANDA_AUTH_USER_PASSWORD,
                                "mechanism": REDPANDA_AUTH_MECHANISM,
                            }
                        ],
                    }
                },
            },
            skip_await=True,
        ),
    )
    return redpanda



import os

from locust import HttpUser, between, task

access_token_cookie_key = os.getenv(
    "PUBLIC_AUTH_ACCESS_TOKEN_COOKIE_KEY", "access_token"
)
admin_username = os.getenv("APP_AUTH_ADMIN_USERNAME", "root")
admin_password = os.getenv("APP_AUTH_ADMIN_PASSWORD", "toor")


class QuickstartUser(HttpUser):
    wait_time = between(1, 5)

    @task(2)
    def homepage(self):
        self.client.get("/")

    @task
    def permissions(self):
        self.client.get("/api/v1/auth/permissions")

    def on_start(self):
        login_response = self.client.post(
            "/api/v1/auth/login",
            json={"identity": admin_username, "password": admin_password},
        )
        login_response_json = login_response.json()
        access_token = login_response_json.get("access_token")
        self.client.cookies.set(access_token_cookie_key, access_token)
